@@ -4,7 +4,7 @@
  *
  * File: Main.java
  *
- * Copyright (c) 2003, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, Static Free Software. All rights reserved.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@ import java.io.PrintStream;
 import java.io.PrintWriter;
 import java.net.URL;
 import java.util.ArrayList;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.prefs.Preferences;
@@ -149,27 +150,27 @@ public final class Main
         if (hasCommandLineOption(argsList, "-help"))
 		{
 	        System.out.println("Usage (without plugins):");
-	        System.out.println("\tjava -jar electric.jar [electric-options]");
+	        System.out.println("\tjava -jar electric.jar [electric-options] [electric-libraries]");
+	        System.out.println("\t\twhere [electric-libraries] is list of library files to read");
 	        System.out.println("Usage (with plugins):");
-	        System.out.println("\tjava -classpath electric.jar<delim>{list of plugins} com.sun.electric.Launcher [electric-options]");
-	        System.out.println("\t\twhere <delim> is OS-dependant separator (colon or semicolon)");
-	        System.out.println("\nOptions:");
+	        System.out.println("\tjava -classpath electric.jar[electric-plugins] com.sun.electric.Launcher [electric-options] [electric-libraries]");
+	        System.out.println("\t\twhere [electric-plugins] is list of JAR files separated by OS-dependant separator (colon or semicolon)");
+	        System.out.println("\nElectric-options:");
+	        System.out.println("\t-batch: batch mode implies 'no GUI', and nothing more");
+            System.out.println("\t-client <machine name>: replay trace of snapshots");
+	        System.out.println("\t-debug: debug mode. Extra information is available");
+	        System.out.println("\t-help: this message");
+            System.out.println("\t-logging <filePath>: log server events in a binary file");
+            System.out.println("\t-logusage <local path>: local path to log usage");
             System.out.println("\t-mdi: multiple document interface mode");
-	        System.out.println("\t-sdi: single document interface mode");
 	        System.out.println("\t-NOMINMEM: ignore minimum memory provided for JVM");
 	        System.out.println("\t-s <script name>: bean shell script to execute");
+	        System.out.println("\t-sdi: single document interface mode");
+            System.out.println("\t-server: dump trace of snapshots");
+            System.out.println("\t-socket <socket>: socket port for client/server interaction");
+            System.out.println("\t-threads <numThreads>: recommended size of thread pool for Job execution.");
 	        System.out.println("\t-version: version information");
 	        System.out.println("\t-v: brief version information");
-	        System.out.println("\t-debug: debug mode. Extra information is available");
-            System.out.println("\t-threads <numThreads>: recommended size of thread pool for Job execution.");
-            System.out.println("\t-logging <filePath>: log server events in a binary file");
-            System.out.println("\t-socket <socket>: socket port for client/server interaction");
-	        System.out.println("\t-batch: batch mode implies 'no GUI', and nothing more");
-            System.out.println("\t-server: dump trace of snapshots");
-            System.out.println("\t-client <machine name>: replay trace of snapshots");
-            System.out.println("\t-logusage <local path>: local path to log usage");
-	        System.out.println("\t-help: this message");
-
 			System.exit(0);
 		}
 
@@ -595,13 +596,28 @@ public final class Main
             if (arg.indexOf('.')!=-1 && SimulationData.isKnownSimulationFormatExtension(arg.substring(arg.lastIndexOf('.')+1))) {
                 SimulationData.plot(null, url, null);
             } else {
-                FileMenu.openLibraryCommand(url);
+            	// make sure library isn't already there
+            	boolean alreadyThere = false;
+//System.out.println("CHECKING REDUNDANCY FOR LIBRARY "+url.toString());
+            	for(Iterator<Library> it = Library.getLibraries(); it.hasNext(); )
+            	{
+            		Library lib = it.next();
+            		URL lURL = lib.getLibFile();
+//System.out.println("   FOUND LIBRARY "+lib.getName()+" ("+lURL+")");
+            		if (lURL != null && lURL.equals(url))
+            		{
+//System.out.println("SKIPPING "+url);		// TODO: debug
+            			alreadyThere = true;  break;
+            		}
+            	}
+            	if (!alreadyThere)
+            		FileMenu.openLibraryCommand(url);
             }
         }
     }
 
 	/**
-	 * Class to init all technologies.
+	 * Class to initialize all technologies.
 	 */
 	private static class InitDatabase extends Job
 	{
