@@ -3,9 +3,9 @@
  * Electric(tm) VLSI Design System
  *
  * File: Netlist.java
- * Written by: Dmitry Nadezhin, Sun Microsystems.
+ * Written by: Dmitry Nadezhin.
  *
- * Copyright (c) 2003, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, Static Free Software. All rights reserved.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -955,20 +955,22 @@ public abstract class Netlist {
             return false;
         }
         if (no1 instanceof IconNodeInst || no2 instanceof IconNodeInst) {
-            int portWidth1 = pp1.getNameKey().busWidth();
-            int portWidth2 = pp2.getNameKey().busWidth();
+            Name portName1 = pp1.getNameKey();
+            Name portName2 = pp2.getNameKey();
+            int portWidth1 = portName1.busWidth();
+            int portWidth2 = portName2.busWidth();
             for (int i = 0; i < busWidth1; i++) {
                 int netIndex1;
                 if (no1 instanceof IconNodeInst) {
-                    netIndex1 = getNetIndex(((IconNodeInst) no1).getNodable(i / portWidth1), pp1, i % portWidth1);
+                    netIndex1 = getNetIndex(((IconNodeInst) no1).getNodable(i / portWidth1), portName1.subname(i % portWidth1));
                 } else {
                     netIndex1 = getNetIndex(no1, pp1, i);
                 }
                 int netIndex2;
                 if (no2 instanceof IconNodeInst) {
-                    netIndex2 = getNetIndex(((IconNodeInst) no2).getNodable(i / portWidth2), pp2, i % portWidth2);
+                    netIndex2 = getNetIndex(((IconNodeInst) no2).getNodable(i / portWidth2), portName2.subname(i % portWidth2));
                 } else {
-                    netIndex2 = getNetIndex(no1, pp1, i);
+                    netIndex2 = getNetIndex(no2, pp2, i);
                 }
                 if (netIndex1 != netIndex2) {
                     return false;
