@@ -4,7 +4,7 @@
  *
  * File: Variable.java
  *
- * Copyright (c) 2003, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, Static Free Software. All rights reserved.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -876,6 +876,9 @@ public class Variable implements Serializable {
         if (name.equals("ART_message")) {
             return "Annotation text";
         }
+        if (name.equals("GDS_text")) {
+            return "Text for GDS output";
+        }
         if (name.equals("GEN_routing_exclusion")) {
             return "Routing exclusion layers";
         }
