@@ -4,7 +4,7 @@
  *
  * File: ToolSettings.java
  *
- * Copyright (c) 2003, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, Static Free Software. All rights reserved.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -169,8 +169,18 @@ public class ToolSettings extends AbstractToolSettings {
 	 */
 	public static Setting getGDSInputScaleSetting() { return t.cacheGDSInputScale; }
     /**
-	 * Method to set the scale to be applied when writing GDS.
-	 * @return the scale to be applied when writing GDS.
+	 * Returns the scale to be applied when reading GDS.
+	 * @return the scale to be applied when reading GDS.
+	 */
+	public static Setting getGDSOutputPrecisionSetting() { return t.cacheGDSOutputPrecision; }
+    /**
+	 * Returns the precision to use when writing GDS.
+	 * @return the precision to use when writing GDS.
+	 */
+	public static Setting getGDSOutputUnitsPerMeterSetting() { return t.cacheGDSOutputUnitsPerMeter; }
+    /**
+	 * Returns the number of units per meter to use when writing GDS.
+	 * @return the number of units per meter to use when writing GDS.
 	 */
 	public static Setting getGDSOutputScaleSetting() { return t.cacheGDSOutputScale; }
     /**
@@ -300,7 +310,9 @@ public class ToolSettings extends AbstractToolSettings {
 	private final Setting cacheGDSOutputConvertsBracketsInExports = makeBooleanSetting("GDSOutputConvertsBracketsInExports", "GDS tab", "GDS output converts brackets in exports", true);
 	private final Setting cacheGDSCellNameLenMax = makeIntSetting("GDSCellNameLenMax", "GDS tab", "GDS name length limit", 32);
     private final Setting cacheGDSInputScale = makeDoubleSetting("GDSInputScale", "GDS tab", "GDS input scale", 1.0);
-    private final Setting cacheGDSOutputScale = makeDoubleSetting("GDSOutputScale", "GDS tab", "GDS output scale", 1.0);
+    private final Setting cacheGDSOutputPrecision = makeDoubleSetting("GDSOutputPrecision", "GDS tab", "GDS output precision", 1000.0);
+    private final Setting cacheGDSOutputUnitsPerMeter = makeDoubleSetting("GDSOutputUnitsPerMeter", "GDS tab", "GDS output units per meter", 1000000000.0);
+        private final Setting cacheGDSOutputScale = makeDoubleSetting("GDSOutputScale", "GDS tab", "GDS output scale", 1.0);
 	private final Setting cacheDXFScale = makeIntSetting("DXFScale", "DXF tab", "DXF scale factor", 2);
 
     { tool("logeffortTool", "tool/logicaleffort"); }
