@@ -4,7 +4,7 @@
  *
  * File: ERCWellCheck.java
  *
- * Copyright (c) 2004, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2004, Static Free Software. All rights reserved.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 package com.sun.electric.tool.erc;
 
 import com.sun.electric.database.geometry.EPoint;
+import com.sun.electric.database.geometry.ERectangle;
 import com.sun.electric.database.geometry.Poly;
 import com.sun.electric.database.hierarchy.Cell;
 import com.sun.electric.database.hierarchy.Export;
@@ -70,7 +71,7 @@ import com.sun.electric.tool.util.concurrent.patterns.PTask;
 import com.sun.electric.tool.util.concurrent.runtime.Scheduler.UnknownSchedulerException;
 import com.sun.electric.tool.util.concurrent.runtime.taskParallel.IThreadPool;
 import com.sun.electric.tool.util.concurrent.runtime.taskParallel.ThreadPool;
-import com.sun.electric.tool.util.concurrent.runtime.taskParallel.ThreadPoolScalaForkJoin;
+import com.sun.electric.tool.util.concurrent.runtime.taskParallel.ThreadPoolJdkForkJoin;
 import com.sun.electric.tool.util.concurrent.utils.BlockedRange1D;
 import com.sun.electric.util.CollectionFactory;
 import com.sun.electric.util.ElapseTimer;
@@ -215,6 +216,9 @@ public class ERCWellCheck {
         @BooleanPref(node = PREF_NODE, key = "FindWorstCaseWell", factory = false)
         public boolean findWorstCaseWell;
 
+        /** true to prevent popups at the end of a run */
+        public boolean disablePopups = false;
+
         public WellCheckPreferences(boolean factory) {
             super(factory);
         }
@@ -240,23 +244,6 @@ public class ERCWellCheck {
         }
     }
 
-//    /**
-//     * Method to analyze the current Cell for well errors.
-//     */
-//    @Deprecated
-//    public static void analyzeCurCell(GeometryHandler.GHMode newAlgorithm) {
-//        UserInterface ui = Job.getUserInterface();
-//        Cell curCell = ui.needCurrentCell();
-//        if (curCell == null) return;
-//
-//        View view = curCell.getView();
-//        if (view.isTextView() || view == View.SCHEMATIC || view == View.ICON) {
-//            System.out.println("Sorry, Well checking runs only on layout cells");
-//            return;
-//        }
-//        new WellCheckJob(curCell, newAlgorithm, new WellCheckPreferences(false));
-//    }
-
     public static void analyzeCurCell() {
         UserInterface ui = Job.getUserInterface();
         Cell curCell = ui.needCurrentCell();
@@ -271,32 +258,11 @@ public class ERCWellCheck {
         new WellCheckJob(curCell, new WellCheckPreferences(false));
     }
 
-//    /**
-//     * Method used by the regressions.
-//     * @param cell the Cell to well-check.
-//     * @param newAlgorithm the geometry algorithm to use.
-//     * @return the success of running well-check.
-//     */
-//    @Deprecated
-//    public static int checkERCWell(Cell cell, GeometryHandler.GHMode newAlgorithm, WellCheckPreferences wellPrefs) {
-//        ERCWellCheck check = new ERCWellCheck(cell, null, newAlgorithm, wellPrefs);
-//        return check.runNow();
-//    }
-
     public static int checkERCWell(Cell cell, WellCheckPreferences wellPrefs) {
         ERCWellCheck check = new ERCWellCheck(cell, null, wellPrefs);
         return check.runNow();
     }
 
-//    @Deprecated
-//    private ERCWellCheck(Cell cell, WellCheckJob job, GeometryHandler.GHMode newAlgorithm,
-//            WellCheckPreferences wellPrefs) {
-//        this.job = job;
-//        this.cell = cell;
-//        this.wellPrefs = wellPrefs;
-//        this.transistors = new HashMap<Integer, List<Transistor>>();
-//    }
-
     private ERCWellCheck(Cell cell, WellCheckJob job, WellCheckPreferences wellPrefs) {
         this.job = job;
         this.cell = cell;
@@ -311,14 +277,6 @@ public class ERCWellCheck {
         private EPoint worstNWellCon, worstNWellEdge;
         private WellCheckPreferences wellPrefs;
 
-//        @Deprecated
-//        private WellCheckJob(Cell cell, GeometryHandler.GHMode newAlgorithm, WellCheckPreferences wellPrefs) {
-//            super("ERC Well Check on " + cell, ERC.tool, Job.Type.SERVER_EXAMINE, null, null, Job.Priority.USER);
-//            this.cell = cell;
-//            this.wellPrefs = wellPrefs;
-//            startJob();
-//        }
-
         private WellCheckJob(Cell cell, WellCheckPreferences wellPrefs) {
             super("ERC Well Check on " + cell, ERC.tool, Job.Type.SERVER_EXAMINE, null, null,
                     Job.Priority.USER);
@@ -408,7 +366,6 @@ public class ERCWellCheck {
                 }
             }
         }
-        // int errorCount = doOldWay();
         int errorCount = doNewWay();
         showStatistics();
 
@@ -417,8 +374,8 @@ public class ERCWellCheck {
         if (errorCount == 0) {
             System.out.println("No Well errors found (took " + timer + ")");
         } else {
-        	if (Job.getDebug())
-        		this.errorLogger.exportErrorLogger(cell.getName()+"logger.xml");
+//        	if (Job.getDebug())
+//        		this.errorLogger.exportErrorLogger(cell.getName()+"logger.xml");
             System.out.println("FOUND " + errorCount + " WELL ERRORS (took " + timer + ")");
 
         }
@@ -455,13 +412,12 @@ public class ERCWellCheck {
         for (WellCon wc: wellCons) {
             assert wc.getWellNum() != null;
             RTNode<WellBound> topSearch = Utils.canBeSubstrateTap(wc.getFun()) ? pWellRoot: nWellRoot;
-            Rectangle2D searchArea = new Rectangle2D.Double(wc.getCtr().getX(), wc.getCtr().getY(), 0, 0);
+            Rectangle2D searchArea = new Rectangle2D.Double(wc.getBound().getCenterX(), wc.getBound().getCenterY(), 0, 0);
             for (Iterator<WellBound> it = new RTNode.Search<WellBound>(searchArea, topSearch, true); it.hasNext(); ) {
                 WellBound wb = it.next();
                 cons++;
                 if (wc.getWellNum().getIndex() != wb.netID.getIndex()) {
                     badCons++;
-//                    System.out.println("??????? checkSpreadResults wc=" + wc.getCtr() + " "  + wc.getWellNum().getIndex() + " wb=" + wb.bound + " " + wb.netID.getIndex());
                 }
             }
         }
@@ -483,7 +439,6 @@ public class ERCWellCheck {
                 pairs++;
                 if (wb.netID.getIndex() != wb2.netID.getIndex()) {
                     badPairs++;
-//                    System.out.println("??????? checkSpreadResults wb=" + wb.bound + " "  + wb.netID.getIndex() + " wb2=" + wb2.bound + " " + wb2.netID.getIndex());
                 }
             }
         }
@@ -499,7 +454,7 @@ public class ERCWellCheck {
                 break;
 
             // see if this contact is a marked well
-            Rectangle2D searchArea = new Rectangle2D.Double(wc.getCtr().getX(), wc.getCtr().getY(), 0, 0);
+            Rectangle2D searchArea = new Rectangle2D.Double(wc.getBound().getCenterX(), wc.getBound().getCenterY(), 0, 0);
             RTNode<WellBound> topSearch = nWellRoot;
             if (Utils.canBeSubstrateTap(wc.getFun()))
                 topSearch = pWellRoot;
@@ -523,12 +478,12 @@ public class ERCWellCheck {
                 String errorMsg = "N-Well contact is floating";
                 if (Utils.canBeSubstrateTap(wc.getFun()))
                     errorMsg = "P-Well contact is floating";
-                errorLogger.logError(errorMsg, EPoint.fromLambda(wc.getCtr().getX(), wc.getCtr().getY()), cell, 0);
+                errorLogger.logError(errorMsg, EPoint.fromLambda(wc.getBound().getCenterX(), wc.getBound().getCenterY()), cell, 0);
                 continue;
             }
 
             // spread out through the well area
-            Utils.spreadWellSeed(wc.getCtr().getX(), wc.getCtr().getY(), wc.getWellNum(), topSearch,
+            Utils.spreadWellSeed(wc.getBound().getCenterX(), wc.getBound().getCenterY(), wc.getWellNum(), topSearch,
                     threadIndex);
         }
     }
@@ -682,11 +637,10 @@ public class ERCWellCheck {
         {
             IThreadPool.NUM_THREADS = Integer.valueOf(numberOfThreads);
             try {
-                threadPool = SCALA_FORKJOIN ? ThreadPoolScalaForkJoin.initialize(numberOfThreads) : ThreadPool.initialize();
+                threadPool = SCALA_FORKJOIN ? ThreadPoolJdkForkJoin.initialize(numberOfThreads) : ThreadPool.initialize();
             } catch (PoolExistsException e) {
             } catch (UnknownSchedulerException e) {
             }
-    //        threadPool = Configuration.lookup(IThreadPool.class);
 
             timer.start();
 
@@ -747,7 +701,6 @@ public class ERCWellCheck {
         // create analysis steps
         List<WellCheckAnalysisStrategy> analysisParts = CollectionFactory.createArrayList();
         analysisParts.add(new ShortCircuitCheck(parameter));
-        // TODO: this analysis can get into an infinite loop in OnRailCheck.createTransistorRec()
         analysisParts.add(new OnRailCheck(parameter, networkExportAvailable, transistors));
         analysisParts.add(new ConnectionCheck(parameter, hasPCon, hasNCon, pWellRoot, nWellRoot, connectedNetValues));
         analysisParts.add(new DRCCheck(parameter, pWellLayer, nWellLayer, pWellRoot, nWellRoot));
@@ -762,6 +715,7 @@ public class ERCWellCheck {
         timer.end();
         System.out.println("   Additional analysis took " + timer);
 
+		if (wellPrefs.disablePopups) errorLogger.disablePopups();
         errorLogger.termLogging(true);
         int errorCount = errorLogger.getNumErrors();
 
@@ -792,7 +746,7 @@ public class ERCWellCheck {
                 if (ind++ % numTasks != taskIndex) continue;
                 assert wc.getWellNum() == null;
                 RTNode<WellBound> topSearch = Utils.canBeSubstrateTap(wc.getFun()) ? pWellRoot: nWellRoot;
-                Rectangle2D searchArea = new Rectangle2D.Double(wc.getCtr().getX(), wc.getCtr().getY(), 0, 0);
+                Rectangle2D searchArea = new Rectangle2D.Double(wc.getBound().getCenterX(), wc.getBound().getCenterY(), 0, 0);
                 for (Iterator<WellBound> it = new RTNode.Search<WellBound>(searchArea, topSearch, true); it.hasNext(); ) {
                     WellBound wb = it.next();
                     result.conBound.put(wc, wb);
@@ -801,7 +755,7 @@ public class ERCWellCheck {
                 String errorMsg = "N-Well contact is floating";
                 if (Utils.canBeSubstrateTap(wc.getFun()))
                     errorMsg = "P-Well contact is floating";
-                errorLogger.logError(errorMsg, EPoint.fromLambda(wc.getCtr().getX(), wc.getCtr().getY()), cell, 0);
+                errorLogger.logError(errorMsg, EPoint.fromLambda(wc.getBound().getCenterX(), wc.getBound().getCenterY()), cell, 0);
                 wc.setWellNum(new NetValues());
             }
             assert ind == wellCons.size();
@@ -833,8 +787,8 @@ public class ERCWellCheck {
     }
     
     /**
-     * Init equivalence map.
-     * @param size numeber of elements in equivalence map
+     * Initialize equivalence map.
+     * @param size number of elements in equivalence map
      * @return integer array representing equivalence map consisting of disjoint elements.
      */
     static int[] initMap(int size) {
@@ -917,15 +871,11 @@ public class ERCWellCheck {
             return cell;
         }
 
-        // public ErrorLogger getErrorLogger() {
-        // return errorLogger;
-        // }
-
         // Apply translation to place the bounding box of the well contact
         // in the correct location on the top cell.
         private Rectangle2D placedWellCon(WellCon wc) {
             Rectangle2D orig = wc.getNi().getBounds();
-            return new Rectangle2D.Double(wc.getCtr().getX() - orig.getWidth() / 2, wc.getCtr().getY()
+            return new Rectangle2D.Double(wc.getBound().getCenterX() - orig.getWidth() / 2, wc.getBound().getCenterY()
                     - orig.getHeight() / 2, orig.getWidth(), orig.getHeight());
         }
 
@@ -939,9 +889,9 @@ public class ERCWellCheck {
                 if (w instanceof WellBound) {
                     list.add(((WellBound) w).getBounds());
                 } else if (w instanceof WellCon) {
-                    // calculate the correct location of WellCon bound with respect to
-                    // the top cell
-                    list.add(placedWellCon(((WellCon) w)));
+                    // calculate the correct location of WellCon bound with respect to the top cell
+                	WellCon wc = (WellCon)w;
+                	list.add(wc.getBound());
                 }
             }
             errorLogger.logMessage(message, list, cell, 0, true);
@@ -972,14 +922,14 @@ public class ERCWellCheck {
                     for (WellCon wc : wellCons) {
                         double dist = 0;
                         if (i == 0)
-                            dist += wc.getCtr().distance(ctr);
+                            dist += wc.getCenter().distance(ctr);
                         else {
                             for (int j = 0; j < i; j++)
-                                dist += wc.getCtr().distance(farPoints[j]);
+                                dist += wc.getCenter().distance(farPoints[j]);
                         }
                         if (dist > farthest) {
                             farthest = dist;
-                            farPoints[i].setLocation(wc.getCtr());
+                            farPoints[i].setLocation(wc.getCenter());
                         }
                     }
                 }
@@ -989,7 +939,7 @@ public class ERCWellCheck {
                     double minDist = Double.MAX_VALUE;
                     int threadNum = 0;
                     for (int j = 0; j < numberOfThreads; j++) {
-                        double dist = wc.getCtr().distance(farPoints[j]);
+                        double dist = wc.getCenter().distance(farPoints[j]);
                         if (dist < minDist) {
                             minDist = dist;
                             threadNum = j;
@@ -1032,13 +982,6 @@ public class ERCWellCheck {
             this.dim = dim;
         }
 
-        /*
-         * (non-Javadoc)
-         *
-         * @see
-         * com.sun.electric.tool.util.concurrent.patterns.PForJob.PForTask#execute
-         * (com.sun.electric.tool.util.concurrent.patterns.PForJob.BlockedRange)
-         */
         @Override
         public void execute() {
             for (int i = range.start(); i < range.end(); i++) {
@@ -1054,8 +997,8 @@ public class ERCWellCheck {
     private GridDim calculateBucket(WellCon con, double sizeX, double sizeY) {
         GridDim result = new GridDim();
 
-        result.xDim = (int) ((con.getCtr().getX() - cell.getBounds().getMinX()) / sizeX);
-        result.yDim = (int) ((con.getCtr().getY() - cell.getBounds().getMinY()) / sizeY);
+        result.xDim = (int) ((con.getBound().getCenterX() - cell.getBounds().getMinX()) / sizeX);
+        result.yDim = (int) ((con.getBound().getCenterY() - cell.getBounds().getMinY()) / sizeY);
 
         return result;
     }
@@ -1304,10 +1247,10 @@ public class ERCWellCheck {
                     else
                         tmpNetNum = info.getNetID(net);
 
-                    WellCon wc = new WellCon(ni.getTrueCenter(), tmpNetNum, null, false, false, fun, ni);
-                    FixpTransform trans = ni.rotateOut();
-                    trans.transform(wc.getCtr(), wc.getCtr());
-                    info.getTransformToRoot().transform(wc.getCtr(), wc.getCtr());
+                    ERectangle er = ni.getBounds();
+                    Rectangle2D bounds = new Rectangle2D.Double(er.getMinX(), er.getMinY(), er.getWidth(), er.getHeight());
+                    info.getTransformToRoot().transform(bounds, bounds);
+                    WellCon wc = new WellCon(bounds, tmpNetNum, null, false, false, fun, ni);
 
                     if (net != null) {
                         // PWell: must be on ground or NWell: must be on power
