@@ -4,7 +4,7 @@
  *
  * File: Input.java
  *
- * Copyright (c) 2003, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, Static Free Software. All rights reserved.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -157,6 +157,7 @@ public class Input<ResultType>
 		{
 			// clean up
 			stopProgressDialog();
+			if (prefs != null && prefs.disablePopups) errorLogger.disablePopups();
 			errorLogger.termLogging(true);
 		}
 
@@ -207,6 +208,8 @@ public class Input<ResultType>
 
 	public abstract static class InputPreferences implements Serializable
 	{
+		public boolean disablePopups = false;
+
 		protected InputPreferences(boolean factory)
 		{
 			if (!factory && !SwingUtilities.isEventDispatchThread() && !Main.isBatch())
