@@ -5,9 +5,9 @@
  * File: GDS.java
  * Input/output tool: GDS output
  * Original C Code written by Sid Penstone, Queens University
- * Translated to Java by Steven M. Rubin, Sun Microsystems.
+ * Translated to Java by Steven M. Rubin.
  *
- * Copyright (c) 2003, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, Static Free Software. All rights reserved.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,23 +31,22 @@ import com.sun.electric.database.hierarchy.Export;
 import com.sun.electric.database.hierarchy.Nodable;
 import com.sun.electric.database.hierarchy.View;
 import com.sun.electric.database.prototype.PortCharacteristic;
-import com.sun.electric.database.prototype.PortOriginal;
 import com.sun.electric.database.prototype.PortProto;
 import com.sun.electric.database.topology.ArcInst;
 import com.sun.electric.database.topology.Geometric;
 import com.sun.electric.database.topology.NodeInst;
-import com.sun.electric.database.topology.PortInst;
 import com.sun.electric.database.variable.UserInterface;
 import com.sun.electric.database.variable.VarContext;
 import com.sun.electric.database.variable.Variable;
+import com.sun.electric.database.variable.Variable.Key;
 import com.sun.electric.technology.ArcProto;
 import com.sun.electric.technology.GDSLayers;
+import com.sun.electric.technology.GDSLayers.GDSLayerType;
 import com.sun.electric.technology.Layer;
 import com.sun.electric.technology.PrimitiveNode;
 import com.sun.electric.technology.Technology;
-import com.sun.electric.technology.GDSLayers.GDSLayerType;
-import com.sun.electric.technology.technologies.Generic;
 import com.sun.electric.technology.technologies.Artwork;
+import com.sun.electric.technology.technologies.Generic;
 import com.sun.electric.tool.Job;
 import com.sun.electric.tool.io.GDSReader;
 import com.sun.electric.tool.io.IOTool;
@@ -88,7 +87,13 @@ import java.util.TreeSet;
  */
 public class GDS extends Geometry
 {
-	/** key of Variable holding high voltage to use. */	public static final Variable.Key HIGH_VOLTAGE_KEY = Variable.newKey("ATTR_high_voltage");
+	private static final boolean NEWUNITS = true;
+
+	/** key of Variable with high voltage text. */		public static final Variable.Key GDS_TEXT_HV_KEY = Variable.newKey("ATTR_GDS_text_HV");
+	/** key of Variable with GDS text text. */			public static final Variable.Key GDS_TEXT_KEY = Variable.newKey("ATTR_GDS_text");
+
+	/** old key of Variable with high voltage text. */	public static final Variable.Key OLD_HIGH_VOLTAGE_KEY = Variable.newKey("ATTR_high_voltage");
+	/** old key of Variable with GDS text text. */		public static final Variable.Key OLD_GDS_TEXT_KEY = Variable.newKey("GDS_text");
 
 	private static final int GDSVERSION        =      3;
 	private static final int BYTEMASK          =   0xFF;
@@ -137,8 +142,6 @@ public class GDS extends Geometry
 	private static final short HDR_N_MAG       =     12;
 
 	// Maximum string sizes
-	//private static final int HDR_M_SNAME       =     32;
-	//private static final int HDR_M_STRNAME     =     32; // replace by preference IOTool.getGDSCellNameMaxLen
 	private static final int HDR_M_ASCII       =    256;
 
 	/** for buffering output data */			private static byte [] dataBufferGDS = new byte[DSIZE];
@@ -147,6 +150,7 @@ public class GDS extends Geometry
 	/** Position of next byte in the buffer */	private static int bufferPosition;
 	/** Number data buffers output so far */	private static int blockCount;
 	/** constant for GDS units */				private double scaleFactor;
+	/** true if rounding caused inaccuracies */	private int inaccurate;
 	/** cell naming map */						private Map<Cell,String> cellNames;
 	/** cells that have been written */			private Set<Cell> writtenCells;
 	/** cell names that have been written */	private Set<String> writtenCellNames;
@@ -168,7 +172,6 @@ public class GDS extends Geometry
 		boolean outMergesBoxes = IOTool.isGDSOutMergesBoxes();
 		public int cellNameLenMax = IOTool.getGDSCellNameLenMax();
 		public boolean outUpperCase = IOTool.isGDSOutUpperCase();
-        double outputScale = IOTool.getGDSOutputScale();
         
         boolean includeText;
 		boolean convertNCCExportsConnectedByParentPins;
@@ -176,6 +179,7 @@ public class GDS extends Geometry
         public boolean flatDesign;
         boolean onlyVisibleLayers;
         boolean[] visibility;
+        double precision, unitsPerMeter;
 
         public GDSPreferences(boolean factory, Cell cell)
 		{
@@ -185,6 +189,8 @@ public class GDS extends Geometry
 				writeAllCells = IOTool.isFactoryGDSWritesEntireLibrary();
 				flatDesign = IOTool.isFactoryGDSFlatDesign();
 				includeText = IOTool.isFactoryGDSIncludesText();
+				precision = IOTool.getFactoryGDSOutputPrecision();
+				unitsPerMeter = IOTool.getFactoryGDSOutputUnitsPerMeter();
 				convertNCCExportsConnectedByParentPins = IOTool.getFactoryGDSConvertNCCExportsConnectedByParentPins();
 				onlyVisibleLayers = IOTool.isFactoryGDSOnlyInvisibleLayers();
 			} else
@@ -192,6 +198,8 @@ public class GDS extends Geometry
 				writeAllCells = IOTool.isGDSWritesEntireLibrary();
 				flatDesign = IOTool.isGDSFlatDesign();
 				includeText = IOTool.isGDSIncludesText();
+				precision = IOTool.getGDSOutputPrecision();
+				unitsPerMeter = IOTool.getGDSOutputUnitsPerMeter();
 				convertNCCExportsConnectedByParentPins = IOTool.getGDSConvertNCCExportsConnectedByParentPins();
 				onlyVisibleLayers = IOTool.isGDSOnlyInvisibleLayers();
 			}
@@ -268,6 +276,12 @@ public class GDS extends Geometry
 
 	protected void done()
 	{
+		if (inaccurate > 0)
+		{
+			String msg = "WARNING: GDS Export encountered problems because of small feature sizes and coarse accuracy settings.";
+			msg += " It is recommended that the 'Units/meter' GDS preference be increased by a factor of " + inaccurate;
+			Job.getUserInterface().showInformationMessage(msg, "Potential GDS Export Problem");
+		}
 	}
 
 	/** Method to write cellGeom */
@@ -391,7 +405,7 @@ public class GDS extends Geometry
 					for(Technology.ArcLayer al : ap.getArcLayers())
 					{
 						Layer layer = al.getLayer();
-						// skip the export if main layer in not visible
+						// skip the export if main layer is not visible
 						if (skipLayer(layer, false)) continue;
 						selectLayer(layer);
                         writeExport(pp, portName, portPoly.getCenterX(), portPoly.getCenterY());
@@ -455,18 +469,21 @@ public class GDS extends Geometry
         // include voltage for nodes and arcs
         for (Iterator<NodeInst> it = cell.getNodes(); it.hasNext(); ) {
             NodeInst ni = it.next();
-            writeHighVoltageText(ni, ni.getAnchorCenterX(), ni.getAnchorCenterY());
+            writeSpecialText(ni, ni.getAnchorCenterX(), ni.getAnchorCenterY(), GDS_TEXT_HV_KEY, OLD_HIGH_VOLTAGE_KEY, GDSLayerType.HIGHVOLTAGE);
+            writeSpecialText(ni, ni.getAnchorCenterX(), ni.getAnchorCenterY(), GDS_TEXT_KEY, OLD_GDS_TEXT_KEY, GDSLayerType.TEXT);
         }
         for (Iterator<ArcInst> it = cell.getArcs(); it.hasNext(); ) {
             ArcInst ai = it.next();
-            writeHighVoltageText(ai, ai.getTrueCenterX(), ai.getTrueCenterY());
+            writeSpecialText(ai, ai.getTrueCenterX(), ai.getTrueCenterY(), GDS_TEXT_HV_KEY, OLD_HIGH_VOLTAGE_KEY, GDSLayerType.HIGHVOLTAGE);
+            writeSpecialText(ai, ai.getTrueCenterX(), ai.getTrueCenterY(), GDS_TEXT_KEY, OLD_GDS_TEXT_KEY, GDSLayerType.TEXT);
         }
         outputHeader(HDR_ENDSTR, 0);
 	}
 
-	private void writeHighVoltageText(Geometric geom, double x, double y)
+	private void writeSpecialText(Geometric geom, double x, double y, Key key, Key keyOld, GDSLayerType type)
 	{
-        Variable var = geom.getVar(HIGH_VOLTAGE_KEY);
+        Variable var = geom.getVar(key);
+        if (var == null) var = geom.getVar(keyOld);
         if (var != null)
         {
         	List<Layer> layers = new ArrayList<Layer>();
@@ -493,10 +510,10 @@ public class GDS extends Geometry
                     continue;
                 }
                 int layerNum = -1, layerType = -1;
-	            if (currentLayerNumbers.hasLayerType(GDSLayerType.HIGHVOLTAGE))
+	            if (currentLayerNumbers.hasLayerType(type))
 	            {
-	            	layerNum = currentLayerNumbers.getLayerNumber(GDSLayerType.HIGHVOLTAGE);
-	            	layerType = currentLayerNumbers.getLayerType(GDSLayerType.HIGHVOLTAGE);
+	            	layerNum = currentLayerNumbers.getLayerNumber(type);
+	            	layerType = currentLayerNumbers.getLayerType(type);
 	            } else
 	            {
 	            	System.out.println("Skipping voltage variable " + layer + " in GDS output");
@@ -545,7 +562,8 @@ public class GDS extends Geometry
 
 				Point2D pt = new Point2D.Double(0, 0);
 				trans.transform(ni.getAnchorCenter(), pt);
-				writeHighVoltageText(ni, pt.getX(), pt.getY());
+				writeSpecialText(ni, pt.getX(), pt.getY(), GDS_TEXT_HV_KEY, OLD_HIGH_VOLTAGE_KEY, GDSLayerType.HIGHVOLTAGE);
+				writeSpecialText(ni, pt.getX(), pt.getY(), GDS_TEXT_KEY, OLD_GDS_TEXT_KEY, GDSLayerType.TEXT);
 			}
 		}
 		for(Iterator<ArcInst> it = cell.getArcs(); it.hasNext(); )
@@ -569,7 +587,8 @@ public class GDS extends Geometry
 
 			Point2D pt = new Point2D.Double(0, 0);
 			trans.transform(ai.getTrueCenter(), pt);
-			writeHighVoltageText(ai, pt.getX(), pt.getY());
+			writeSpecialText(ai, pt.getX(), pt.getY(), GDS_TEXT_HV_KEY, OLD_HIGH_VOLTAGE_KEY, GDSLayerType.HIGHVOLTAGE);
+			writeSpecialText(ai, pt.getX(), pt.getY(), GDS_TEXT_KEY, OLD_GDS_TEXT_KEY, GDSLayerType.TEXT);
 		}
 
 		// write exports
@@ -579,11 +598,11 @@ public class GDS extends Geometry
 			{
 				Export pp = eIt.next();
 
-				// find the node at the bottom of this export
-				PortOriginal fp = new PortOriginal(pp.getOriginalPort());
-				PortInst bottomPort = fp.getBottomPort();
-				NodeInst bottomNi = bottomPort.getNodeInst();
-
+//				// find the node at the bottom of this export
+//				PortOriginal fp = new PortOriginal(pp.getOriginalPort());
+//				PortInst bottomPort = fp.getBottomPort();
+//				NodeInst bottomNi = bottomPort.getNodeInst();
+//
 //				// find the layer associated with this node
 //				PrimitiveNode pNp = (PrimitiveNode)bottomNi.getProto();
 //				Technology.NodeLayer [] nLay = pNp.getNodeLayers();
@@ -1104,7 +1123,13 @@ public class GDS extends Geometry
 		for (int i=0; i<DSIZE; i++) emptyBuffer[i] = 0;
 
 		Technology tech = topCell.getTechnology();
-		scaleFactor = tech.getScale() * localPrefs.outputScale;
+		scaleFactor = tech.getScale() * localPrefs.unitsPerMeter / 1000000000.0;
+		inaccurate = 0;
+//		tech.getScale() = 10 (measured in nanometers)
+//			so 210nm is 21 lambda
+//		localPrefs.precision = 1000;
+//		localPrefs.unitsPerMeter = 1,000,000,000
+//			sp 210nm is 21 lambda, so write 21 * unitsPerMeter / 1,000,000,000
 		layerNumbers = new HashMap<Layer,GDSLayers>();
 		nameRemapping = new HashMap<String,Set<String>>();
 
@@ -1281,10 +1306,18 @@ public class GDS extends Geometry
 		 * 0x3F28F5C2,0x8F5C28F6 = 0.01
 		 * 0x3A2AF31D,0xC4611874 = 1e-8
 		 */
-
 		// set units
-		outputDouble(1e-3);
-		outputDouble(1.0e-9);
+		if (NEWUNITS)
+		{
+			double userUnitsPerDBUnit = 1.0 / localPrefs.precision;
+			double dbUnitsInMeters = 1.0 / localPrefs.unitsPerMeter;
+			outputDouble(userUnitsPerDBUnit);
+			outputDouble(dbUnitsInMeters);
+		} else
+		{		
+			outputDouble(1e-3);
+			outputDouble(1.0e-9);
+		}
 	}
 
 	boolean outputBeginStruct(Cell cell)
@@ -1571,6 +1604,18 @@ public class GDS extends Geometry
 
 		// round to nearest nanometer
 		int unit = (int)Math.round(scaled);
+		if (unit != scaled)
+		{
+			int accurateScale = 10;
+			for(int i=0; i<10; i++)
+			{
+				double scaledScale = scaled * accurateScale;
+				long scaledUnit = (long)Math.round(scaledScale);
+				if (scaledUnit == scaledScale) break;
+				accurateScale *= 10;
+			}
+			if (accurateScale > inaccurate) inaccurate = accurateScale;
+		}
 		return unit;
 	}
 
