@@ -4,9 +4,9 @@
  *
  * File: Telesis.java
  * Input/output tool: Telesis output
- * Written by Gilda Garreton, Oracle Inc.
+ * Written by Gilda Garreton
  *
- * Copyright (c) 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, Static Free Software. All rights reserved.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,14 +42,13 @@ import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
-import java.util.Map.Entry;
+import java.util.Set;
 import java.util.StringTokenizer;
 
-import scala.Array;
-
 public class Telesis extends Geometry {
 
 //	private TelesisPreferences localPrefs;
@@ -95,7 +94,7 @@ public class Telesis extends Geometry {
 	 * @param name
 	 * @param f
 	 * @param l
-	 * @return
+	 * @return root name.
 	 */
 	private static String extractRootNameAndArrayIndices(String name, MutableInteger f, MutableInteger l,
 			List<String> secondArray)
@@ -329,7 +328,6 @@ public class Telesis extends Geometry {
 				return finalName;
 		}
 		System.out.println("Error: it should not reach this level extracting port name: " + rootName + ", " + matchingName);
-		//assert(false); // should it reach this level?
 		return finalName;
 	}
 	
@@ -364,7 +362,7 @@ public class Telesis extends Geometry {
 			if (values.length > 1) // array, single or double
 			{
 				numbers = new String[values.length-1];
-				Array.copy(values, 1, numbers, 0, values.length-1);
+				System.arraycopy(values, 1, numbers, 0, values.length-1);
 			}
 		}
 	}
@@ -431,14 +429,21 @@ public class Telesis extends Geometry {
 			{
 				printWriter.println("$PACKAGES");
 				// list of instances
-				for (Entry<Cell,List<NodeInst>> e : subCellsMap.entrySet())
+				// Get the keys first and sort them so the output will be deterministic.
+				List<Cell> keysList = new ArrayList<Cell>(subCellsMap.keySet());
+				Collections.sort(keysList); // it should sort by cell names;
+				
+				for (Cell c : keysList)
 				{
-					printWriter.print("! " + e.getKey().getName() + " ; ");
-					for (NodeInst ni : e.getValue())
+					List<NodeInst> nlist = subCellsMap.get(c);
+					assert(nlist != null);
+					printWriter.print("! " + c.getName() + " ; ");
+					for (NodeInst ni : nlist)
 						printWriter.print(correctTelesisName(ni.getName()) + " ");
-					printWriter.println(" (# instances " + e.getValue().size() + ")");
+					printWriter.println(" (# instances " + nlist.size() + ")");
 				}
 				printWriter.println();
+				
 				// nets
 				printWriter.println("$NETS");
 				Netlist netlist = cell.getNetlist();
@@ -458,13 +463,13 @@ public class Telesis extends Geometry {
 						String s = sIt.next();  // .nextToken();
 						listConnect.add(new NetBusMatching(s));
 			        }
-
+					
 					int count = 0;
 					StringBuffer netString = new StringBuffer("'" + newString + "' ; ");
 					// iterate along list connect
 					// Old code
 					count = 0;
-					List<PortInst> portsVisited = new ArrayList<PortInst>();
+					Set<PortInst> portsVisited = new HashSet<PortInst>();
 					
 					for (NetBusMatching nbl : listConnect)
 					{
@@ -474,7 +479,7 @@ public class Telesis extends Geometry {
 						{
 							PortInst p = itP.next();
 							if (portsVisited.contains(p)) continue; // already analyzed
-							portsVisited.add(p);
+//							portsVisited.add(p);
 							
 							NodeInst pi = p.getNodeInst();
 							if (!pi.isCellInstance()) continue; // skipping non-subcell instances
@@ -553,7 +558,7 @@ public class Telesis extends Geometry {
 										else
 										{
 											// check that they match in index. Consider single digits or set like p,n
-											if (!values[i].contains(nbl.numbers[i-1]))
+											if (i > nbl.numbers.length || !values[i].contains(nbl.numbers[i-1]))
 											{
 												noMatching = true;
 												break;
@@ -615,6 +620,7 @@ public class Telesis extends Geometry {
 										}
 										netString.append(theFinalName);
 										count += portCount.intValue();
+										portsVisited.add(p);
 										break; // done with this array
 									}
 								}
@@ -634,6 +640,7 @@ public class Telesis extends Geometry {
 									}
 									netString.append(theFinalName);
 									count += portCount.intValue();
+									portsVisited.add(p);
 								}
 								foundPortMatch = true;
 								// if there is only 1 element in listConnect -> forcing to check more connections 
@@ -797,9 +804,7 @@ public class Telesis extends Geometry {
 			int totalNumberOfPins = 0;
 			
 			if (iconCell == null)
-			{
 				System.out.println("No icon cell found for cell '" + cellName + "'");
-			}
 			else
 			{
 				// check if cell has any Allegro variable. If not, it is considered as the top cell
