@@ -4,9 +4,9 @@
  *
  * File: Topology.java
  * Input/output tool: superclass for output modules that write connectivity.
- * Written by Steven M. Rubin, Sun Microsystems.
+ * Written by Steven M. Rubin.
  *
- * Copyright (c) 2003, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, Static Free Software. All rights reserved.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -540,11 +540,58 @@ public abstract class Topology extends Output
 
 			CellSignal cs = new CellSignal();
 			cs.pp = null;
-			cs.descending = !NetworkTool.isBusAscending();
 			cs.power = false;
 			cs.ground = false;
 			cs.net = net;
 
+			// determine ascending/descending nature of this network
+			cs.descending = !NetworkTool.isBusAscending();
+			boolean foundDirection = false;
+			if (net.isExported())
+			{
+				// determine ascending/descending from export
+				for(Iterator<Export> eIt = net.getExports(); eIt.hasNext(); )
+				{
+					Export e = eIt.next();
+					Boolean ascending = isArrayAscending(e.getName());
+					if (ascending != null)
+					{
+						cs.descending = ascending.booleanValue();
+						foundDirection = true;
+						break;
+					}
+				}
+			}
+			if (!foundDirection)
+			{
+				for(Iterator<ArcInst> aIt = net.getArcs(); aIt.hasNext(); )
+				{
+					ArcInst ai = aIt.next();
+					if (!ai.isUsernamed()) continue;
+					Boolean ascending = isArrayAscending(ai.getName());
+					if (ascending != null)
+					{
+						cs.descending = ascending.booleanValue();
+						foundDirection = true;
+						break;
+					}
+				}
+			}
+			if (!foundDirection)
+			{
+				for(Iterator<PortInst> pIt = net.getPorts(); pIt.hasNext(); )
+				{
+					PortInst pi = pIt.next();
+					Boolean ascending = isArrayAscending(pi.getPortProto().getName());
+					if (ascending != null)
+					{
+						cs.descending = ascending.booleanValue();
+						foundDirection = true;
+						break;
+					}
+				}
+			}
+
 			// see if it is global
 			cs.globalSignal = null;
 			for(int j=0; j<globalSize; j++)
@@ -1010,6 +1057,31 @@ public abstract class Topology extends Output
 		return name.substring(0, i);
 	}
 
+	/**
+	 * Method to analyze a name and determine if there are ascending or descending busses in it.
+	 * @param name the name.
+	 * @return TRUE if ascending busses found, FALSE if descending busses found, null if no busses found.
+	 */
+	private static Boolean isArrayAscending(String name)
+	{
+		int startPos = 0;
+		for(;;)
+		{
+			int colonPos = name.indexOf(':', startPos);
+			if (colonPos < 0) break;
+			startPos = colonPos+1;
+			int firstNumPos = colonPos;
+			while (firstNumPos > 0 && Character.isDigit(name.charAt(firstNumPos-1))) firstNumPos--;
+			if (firstNumPos >= colonPos) continue;
+			int firstNum = TextUtils.atoi(name.substring(firstNumPos));
+			if (colonPos+1 >= name.length()) continue;
+			int secondNum = TextUtils.atoi(name.substring(colonPos+1));
+			if (firstNum <= secondNum) return Boolean.FALSE;
+			return Boolean.TRUE;
+		}
+		return null;
+	}
+
 	private static class SortNetsByName implements Comparator<CellSignal>
 	{
 		private boolean separateInputAndOutput, aggregateNamesSupported, caseSensitive, descendingAscendingSupported;
