@@ -4,10 +4,10 @@
  *
  * File: Verilog.java
  * Input/output tool: Verilog Netlist output
- * Written by Steven M. Rubin, Sun Microsystems.
+ * Written by Steven M. Rubin.
  * VerilogA by Min Hao Zhu, Villanova University.
  *
- * Copyright (c) 2003, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, Static Free Software. All rights reserved.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -86,8 +86,7 @@ public class Verilog extends Topology
 	/** key of Variable holding file name with Verilog. */	public static final Variable.Key VERILOG_BEHAVE_FILE_KEY = Variable.newKey("SIM_verilog_behave_file");
 	/** those cells that have overridden models */			private Set<Cell> modelOverrides = new HashSet<Cell>();
 	/** those cells that have modules defined */			private Map<String,String> definedModules = new HashMap<String,String>(); // key: module name, value: source description
-//	/** those cells that have primitives defined */			private Map<Cell,VerilogData.VerilogModule> definedPrimitives = new HashMap<Cell,VerilogData.VerilogModule>(); // key: module name, value: VerilogModule
-	/** those cells that have primitives defined (new) */	private Map<Cell,CompileVerilogStruct.VModule> definedPrimitivesNew = new HashMap<Cell,CompileVerilogStruct.VModule>(); // key: module name, value: VerilogModule
+	/** those cells that have primitives defined */			private Map<Cell,CompileVerilogStruct.VModule> definedPrimitives = new HashMap<Cell,CompileVerilogStruct.VModule>(); // key: module name, value: VerilogModule
 	/** those cells which are empty */						private Map<Cell,Cell> emptyModules = new HashMap<Cell,Cell>();
 	/** map of cells that are or contain standard cells */  private SCLibraryGen.StandardCellHierarchy standardCells = new SCLibraryGen.StandardCellHierarchy();
     /** file we are writing to */                           private String filePath;
@@ -263,12 +262,6 @@ public class Verilog extends Topology
 		if (localPrefs.useVerilogA) reservedWords.add("NCNFET");
 	}
 
-//    public static String getVerilogSafeName(String name, boolean isNode, boolean isBus) {
-//    	Verilog v = new Verilog(new VerilogPreferences());
-//    	if (isNode) return v.getSafeCellName(name);
-//    	return v.getSafeNetName(name, isBus);
-//    }
-
 	protected void start()
 	{
 		// parameters to the output-line-length limit and how to break long lines
@@ -778,7 +771,6 @@ public class Verilog extends Topology
 		if (localPrefs.stopAtStandardCells)
 		{
 			if (SCLibraryGen.isStandardCell(cell)) {
-//				printWriter.println("endmodule   /* " + cellName + " */");
 				writeLine("endmodule   /* " + cellName + " */", moduleStr);
 				return;
 			}
@@ -795,19 +787,6 @@ public class Verilog extends Topology
 			if (cni.getGroundNet() != null) writeLine("  supply0 gnd;", moduleStr); // printWriter.println("  supply0 gnd;");
 		}
 
-    	previouslyDeclared = new HashSet<String>();
-//		// add in any user-specified declarations and code
-//        if (!localPrefs.stopAtStandardCells || localPrefs.netlistNonstandardCells) {
-//            // STA does not like general verilog code (like and #delay out InA InB etc)
-//            first = includeTypedCode(cell, VERILOG_DECLARATION_KEY, "declarations", moduleStr);
-//		    first |= includeTypedCode(cell, VERILOG_CODE_KEY, "code", moduleStr);
-//		    if (!first)
-//		    {
-//		    	String verilogType = localPrefs.useVerilogA ? "VerilogA" : "Verilog";
-//		    	writeLine("  /* automatically generated " + verilogType + " */", moduleStr); // printWriter.println("  /* automatically generated " + verilogType + " */");
-//		    }
-//        }
-
 		// determine whether to use "wire" or "trireg" for networks
 		String wireType;
 		if (localPrefs.useVerilogA)
@@ -821,45 +800,46 @@ public class Verilog extends Topology
 
 		// write "wire/trireg" declarations for internal single-wide signals
 		int localWires = 0;
-		for(int wt=0; wt<2; wt++)
+		Map<String,List<String>> internalSignals = new HashMap<String,List<String>>();
+		for(Iterator<CellAggregateSignal> it = cni.getCellAggregateSignals(); it.hasNext(); )
 		{
-			first = true;
-			for(Iterator<CellAggregateSignal> it = cni.getCellAggregateSignals(); it.hasNext(); )
-			{
-				CellAggregateSignal cas = it.next();
-				if (cas.getExport() != null) continue;
-				if (cas.isSupply()) continue;
-				if (cas.getLowIndex() <= cas.getHighIndex()) continue;
-				if (cas.isGlobal()) continue;
+			CellAggregateSignal cas = it.next();
+			if (cas.getExport() != null) continue;
+			if (cas.isSupply()) continue;
+			if (cas.getLowIndex() <= cas.getHighIndex()) continue;
+			if (cas.isGlobal()) continue;
 
-				String impSigName = wireType;
-				if (cas.getFlags() != 0)
+			String impSigName = wireType;
+			if (cas.getFlags() != 0)
+			{
+				if (localPrefs.useVerilogA)
 				{
-					if (localPrefs.useVerilogA)
-					{
-						impSigName = "electrical";
-					} else
-					{
-						if (cas.getFlags() == 1) impSigName = "wire"; else
-							impSigName = "trireg";
-					}
-				}
-				if ((wt == 0) ^ !wireType.equals(impSigName))
+					impSigName = "electrical";
+				} else
 				{
-					if (previouslyDeclared.contains(cas.getName())) continue;
-					if (first)
-					{
-						initDeclaration("  " + impSigName);
-					}
-					addDeclaration(cas.getName(), moduleStr);
-					localWires++;
-					first = false;
+					if (cas.getFlags() == 1) impSigName = "wire"; else
+						impSigName = "trireg";
 				}
 			}
-			if (!first) termDeclaration(moduleStr);
+			List<String> sigs = internalSignals.get(impSigName);
+			if (sigs == null) internalSignals.put(impSigName, sigs = new ArrayList<String>());
+			if (!sigs.contains(cas.getName()))
+				sigs.add(cas.getName());
+			localWires++;
+		}
+		for(String decType : internalSignals.keySet())
+		{
+			List<String> sigs = internalSignals.get(decType);
+			if (sigs.size() == 0) continue;
+			Collections.sort(sigs, TextUtils.STRING_NUMBER_ORDER);
+			initDeclaration("  " + decType);
+			for(String st : sigs) addDeclaration(st, moduleStr);
+			termDeclaration(moduleStr);
 		}
 
 		// write "wire/trireg" declarations for internal busses
+    	previouslyDeclared = new HashSet<String>();
+		List<String> intBusses = new ArrayList<String>();
 		for(Iterator<CellAggregateSignal> it = cni.getCellAggregateSignals(); it.hasNext(); )
 		{
 			CellAggregateSignal cas = it.next();
@@ -876,21 +856,26 @@ public class Verilog extends Topology
 				{
 					int ind = i;
 					if (cas.isDescending()) ind = indices.length - i - 1;
-					writeLine("  " + wireType + " \\" + cas.getName() + "[" + indices[ind] + "] ;", moduleStr); // printWriter.println("  " + wireType + " \\" + cas.getName() + "[" + indices[ind] + "] ;");
+					intBusses.add("  " + wireType + " \\" + cas.getName() + "[" + indices[ind] + "] ;");
 				}
 			} else
 			{
 				if (cas.isDescending())
 				{
-					writeLine("  " + wireType + " [" + cas.getHighIndex() + ":" + cas.getLowIndex() + "] " + cas.getName() + ";", moduleStr); // printWriter.println("  " + wireType + " [" + cas.getHighIndex() + ":" + cas.getLowIndex() + "] " + cas.getName() + ";");
+					intBusses.add("  " + wireType + " [" + cas.getHighIndex() + ":" + cas.getLowIndex() + "] " + cas.getName() + ";");
 				} else
 				{
-					writeLine("  " + wireType + " [" + cas.getLowIndex() + ":" + cas.getHighIndex() + "] " + cas.getName() + ";", moduleStr); // printWriter.println("  " + wireType + " [" + cas.getLowIndex() + ":" + cas.getHighIndex() + "] " + cas.getName() + ";");
+					intBusses.add("  " + wireType + " [" + cas.getLowIndex() + ":" + cas.getHighIndex() + "] " + cas.getName() + ";");
 				}
 			}
 			localWires++;
 		}
-		if (localWires != 0) writeLine("", moduleStr); // printWriter.println();
+		if (intBusses.size() > 0)
+		{
+			Collections.sort(intBusses, TextUtils.STRING_NUMBER_ORDER);
+			for(String st : intBusses) writeLine(st, moduleStr);
+		}
+		if (localWires != 0) writeLine("", moduleStr);
 
 		// add "wire" declarations for implicit inverters
 		if (impInvCount > 0)
@@ -1135,15 +1120,8 @@ public class Verilog extends Topology
                 }
 
                 // cells defined as "primitives" in Verilog View must have implicit port ordering
-//                if (VerilogReader.NEWVERILOGPARSER)
-//                {
-                	if (definedPrimitivesNew.containsKey(niProto))
-                		implicitPorts = 3;
-//                } else
-//                {
-//					if (definedPrimitives.containsKey(niProto))
-//						implicitPorts = 3;
-//                }
+            	if (definedPrimitives.containsKey(niProto))
+            		implicitPorts = 3;
 			} else
 			{
 				// convert 4-port transistors to 3-port
@@ -1325,19 +1303,6 @@ public class Verilog extends Topology
                                         {
                                         	System.out.println("Warning: Cell " + e.getParent().describe(false) + ", entry " + ind +
                                         		" of port " + e.getName() + " has no equivalent in cell " + altCell.describe(false));
-//System.out.print("   Ports are:");
-//String lastName = "";
-//for(Iterator<CellAggregateSignal> eIt = subCni.getCellAggregateSignals(); eIt.hasNext(); )
-//{
-//	CellAggregateSignal casE = eIt.next();
-//	Export ppE = casE.getExport();
-//	if (ppE == null) continue;
-//	String thisName = ppE.getName();
-//	if (thisName.equals(lastName)) continue;
-//	System.out.print(" " + thisName);
-//	lastName = thisName;
-//}
-//System.out.println();
                                         	continue;
                                         }
                                         e = altE;
@@ -1433,54 +1398,26 @@ public class Verilog extends Topology
 					break;
 				case 3:			// implicit ports ordering for cells defined as primitives in Verilog View
 					ni = no.getNodeInst();
-//	                if (VerilogReader.NEWVERILOGPARSER)
-//	                {
-						CompileVerilogStruct.VModule module = definedPrimitivesNew.get(niProto);
-						if (module == null) break;
-						System.out.print(cell.getName()+" ports: ");
-						for (CompileVerilogStruct.VExport port : module.getPorts()) {
-							if (port.isBus()) {
-								// Bill thinks bussed ports are not allowed for primitives
-								reportError("Error: bussed ports not allowed on Verilog primitives: "+niProto.getName());
-							} else {
-								String portName = port.getName();
-								PortInst pi = ni.findPortInst(portName);
-								Network net = netList.getNetwork(no, pi.getPortProto().getNameKey().subname(0));
-								CellSignal cs = cni.getCellSignal(net);
-								String sigName = getSignalName(cs);
+					CompileVerilogStruct.VModule module = definedPrimitives.get(niProto);
+					if (module == null) break;
+					System.out.print(cell.getName()+" ports: ");
+					for (CompileVerilogStruct.VExport port : module.getPorts()) {
+						if (port.isBus()) {
+							// Bill thinks bussed ports are not allowed for primitives
+							reportError("Error: bussed ports not allowed on Verilog primitives: "+niProto.getName());
+						} else {
+							String portName = port.getName();
+							PortInst pi = ni.findPortInst(portName);
+							Network net = netList.getNetwork(no, pi.getPortProto().getNameKey().subname(0));
+							CellSignal cs = cni.getCellSignal(net);
+							String sigName = getSignalName(cs);
 
-								if (first) first = false; else
-									infstr.append(", ");
-								infstr.append(sigName);
-								System.out.print(portName+" ");
-							}
+							if (first) first = false; else
+								infstr.append(", ");
+							infstr.append(sigName);
+							System.out.print(portName+" ");
 						}
-//	                } else
-//	                {
-//						VerilogData.VerilogModule module = definedPrimitives.get(niProto);
-//						if (module == null) break;
-//						System.out.print(cell.getName()+" ports: ");
-//						for (VerilogData.VerilogPort port : module.getPorts()) {
-//							List<String> portnames = port.getPinNames(true);
-//							if (portnames.size() == 0) {
-//								// continue
-//							} else if (portnames.size() > 1) {
-//								// Bill thinks bussed ports are not allowed for primitives
-//								reportError("Error: bussed ports not allowed on Verilog primitives: "+niProto.getName());
-//							} else {
-//								String portname = portnames.get(0);
-//								PortInst pi = ni.findPortInst(portname);
-//								Network net = netList.getNetwork(no, pi.getPortProto().getNameKey().subname(0));
-//								CellSignal cs = cni.getCellSignal(net);
-//								String sigName = getSignalName(cs);
-//
-//								if (first) first = false; else
-//									infstr.append(", ");
-//								infstr.append(sigName);
-//								System.out.print(portname+" ");
-//							}
-//						}
-//	                }
+					}
 					System.out.println();
 					infstr.append(");");
 					break;
@@ -1707,7 +1644,7 @@ public class Verilog extends Topology
      * @param line
      * @param no
      * @param context
-     * @return
+     * @return the parameter string.
      */
     private String writeDefparam(String line, Nodable no, VarContext context)
     {
@@ -1751,7 +1688,6 @@ public class Verilog extends Topology
         	infstr.append(";");
         }
         infstr.append("\n");
-        // writeWidthLimited(infstr.toString());
         return infstr.toString();
     }
 
@@ -1768,7 +1704,7 @@ public class Verilog extends Topology
 	 * @param varName
 	 * @param no
 	 * @param context
-	 * @return
+	 * @return the Variable string.
 	 */
     private String replaceVariable(String varName, Nodable no, VarContext context)
     {
@@ -1801,7 +1737,7 @@ public class Verilog extends Topology
 	 *
 	 * @param varName
 	 * @param cell
-	 * @return
+	 * @return the Variable string.
 	 */
     private String replaceVariable(String varName, Cell cell)
     {
@@ -1836,7 +1772,7 @@ public class Verilog extends Topology
 	 *
 	 * @param line
 	 * @param cell
-	 * @return
+	 * @return the Variable string.
 	 */
     private String replaceVarInString(String line, Cell cell)
     {
@@ -1995,12 +1931,14 @@ public class Verilog extends Topology
 			{
 				int first = TextUtils.atoi(netName.substring(i+1));
 				int second = TextUtils.atoi(lastNetName.substring(openSquare+1));
-				infstr.append("[" + first + ":" + second + "]");
+//				if (first == second) infstr.append("[" + first + "]"); else
+					infstr.append("[" + first + ":" + second + "]");
 			} else
 			{
 				int first = TextUtils.atoi(netName.substring(i+1));
 				int second = TextUtils.atoi(lastNetName.substring(openSquare+1));
-				infstr.append("[" + second + ":" + first + "]");
+//				if (first == second) infstr.append("[" + first + "]"); else
+					infstr.append("[" + second + ":" + first + "]");
 			}
 		}
 		if (name != null) infstr.append(")");
@@ -2111,7 +2049,6 @@ public class Verilog extends Topology
 		if (sim_verDeclarationLine.length() + signame.length() + 3 > MAXDECLARATIONWIDTH)
 		{
 			writeLine(sim_verDeclarationLine.toString() + ";", buffer);
-//			printWriter.println(sim_verDeclarationLine.toString() + ";");
 			sim_verDeclarationLine.delete(sim_verdeclarationprefix, sim_verDeclarationLine.length());
 		}
 		if (sim_verDeclarationLine.length() != sim_verdeclarationprefix)
@@ -2164,7 +2101,6 @@ public class Verilog extends Topology
 	}
 
 	/** Method to return the proper name of Power */
-//	protected String getPowerName(Network net) { return "vdd"; }
 	protected String getPowerName(Network net)
 	{
 		if (net != null)
@@ -2180,7 +2116,6 @@ public class Verilog extends Topology
 	}
 
 	/** Method to return the proper name of Ground */
-//	protected String getGroundName(Network net) { return "gnd"; }
 	protected String getGroundName(Network net)
 	{
 		if (net != null)
@@ -2331,92 +2266,6 @@ public class Verilog extends Topology
 		"rtranif1",
 	};
 
-//	/**
-//	 * Perform some checks on parsed Verilog data, including the
-//	 * check that module is defined for the cell it is replacing.
-//	 * Some checks are different depending upon if the source is
-//	 * a Verilog View or an included external file.
-//	 * @param data the parsed Verilog data
-//	 * @param cell the cell to be replaced (must be verilog view if replacing Verilog View)
-//	 * @param includeFile the included file (null if replacing Verilog View)
-//	 * @return false if there was an error
-//	 */
-//	private boolean checkIncludedData(VerilogData data, Cell cell, String includeFile) {
-//		// make sure there is module for current cell
-//		Collection<VerilogData.VerilogModule> modules = data.getModules();
-//		VerilogData.VerilogModule main = null, alternative = null;
-//		for (VerilogData.VerilogModule mod : modules) {
-//			if (mod.getName().equals(getVerilogName(cell)))
-//				main = mod;
-//			else {
-//				Cell.CellGroup grp = cell.getCellGroup(); // if cell group is available
-//				if (grp != null && mod.getName().equals(grp.getName()))
-//					alternative = mod;
-//			}
-//		}
-//		if (main == null) main = alternative;
-//		if (main == null) {
-//			reportError("Error! Expected Verilog module definition '"+getVerilogName(cell)+
-//			" in Verilog View: "+cell.libDescribe());
-//			return false;
-//		}
-//        if (main.isPrimitive())
-//            definedPrimitives.put(cell, main);
-//
-//		String source = includeFile == null ? "Verilog View for "+cell.libDescribe() :
-//			"Include file: "+includeFile;
-//		// check that modules have not already been defined
-//		for (VerilogData.VerilogModule mod : modules) {
-//			String prevSource = definedModules.get(mod.getName());
-//			if (mod.isValid()) {
-//				if (prevSource != null) {
-//					reportError("Error, module "+mod.getName()+" already defined from: "+prevSource);
-//				}
-//				else
-//					definedModules.put(mod.getName(), source);
-//			}
-//		}
-//
-//		// make sure ports for module match ports for cell
-////		Collection<VerilogData.VerilogPort> ports = main.getPorts();
-//		// not sure how to do this right now
-//
-//		// check for undefined instances, search libraries for them
-//		for (VerilogData.VerilogModule mod : modules) {
-//			for (VerilogData.VerilogInstance inst : mod.getInstances()) {
-//				VerilogData.VerilogModule instMod = inst.getModule();
-//				if (instMod.isValid())
-//					continue;	   // found in file, continue
-//				// check if primitive
-//				boolean primitiveGate = false;
-//				for (String s : verilogGates) {
-//					if (s.equals(instMod.getName().toLowerCase())) {
-//						primitiveGate = true;
-//						break;
-//					}
-//				}
-//				if (primitiveGate) continue;
-//
-//				// undefined, look in modules already written
-//				String moduleName = instMod.getName();
-//				String found = definedModules.get(moduleName);
-//				if (found == null && includeFile == null) {
-//					// search libraries for it
-//					Cell missingCell = findCell(moduleName, View.VERILOG);
-//					if (missingCell == null) {
-//						reportError("Error: Undefined reference to module "+moduleName+", and no matching cell found");
-//						continue;
-//					}
-//					// hmm...name map might be wrong at for this new enumeration
-//					System.out.println("Info: Netlisting cell "+missingCell.libDescribe()+" as instanced in: "+source);
-//					HierarchyEnumerator.enumerateCell(missingCell, VarContext.globalContext,
-//						new Visitor(this), getShortResistors());
-//				}
-//			}
-//		}
-//		return true;
-//	}
-
 	/**
 	 * Perform some checks on parsed Verilog data, including the
 	 * check that module is defined for the cell it is replacing.
@@ -2448,7 +2297,7 @@ public class Verilog extends Topology
 			return false;
 		}
         if (main.isPrimitive())
-            definedPrimitivesNew.put(cell, main);
+        	definedPrimitives.put(cell, main);
 
 		String source = includeFile == null ? "Verilog View for " + cell.libDescribe() :
 			"Include file: " + includeFile;
