@@ -4,7 +4,7 @@
  *
  * File: LETool.java
  *
- * Copyright (c) 2003, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, Static Free Software. All rights reserved.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,8 +18,6 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Created on November 17, 2003, 10:16 AM
  */
 
 package com.sun.electric.tool.logicaleffort;
@@ -43,11 +41,16 @@ import com.sun.electric.tool.Tool;
 import com.sun.electric.tool.ToolSettings;
 import com.sun.electric.tool.generator.sclibrary.SCLibraryGen;
 import com.sun.electric.tool.lang.EvalJavaBsh;
-import com.sun.electric.tool.user.ui.EditWindow;
 import com.sun.electric.tool.simulation.SimulationTool;
+import com.sun.electric.tool.user.ui.EditWindow;
 import com.sun.electric.util.TextUtils;
 
-import java.util.*;
+import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
@@ -178,7 +181,7 @@ public class LETool extends Tool {
     }
 
     /**
-     * Grab a paramter 'parName' from a nodeInst 'nodeName' in a sub cell.
+     * Grab a parameter 'parName' from a nodeInst 'nodeName' in a sub cell.
      * @param nodeName name of the nodeInst
      * @param parName name of parameter to evaluate
      * @return the parameter.
@@ -188,7 +191,7 @@ public class LETool extends Tool {
         // info should be the node on which there is the variable with the subDrive() call
         Object info = EvalJavaBsh.evalJavaBsh.getCurrentInfo();
         if (!(info instanceof Nodable)) throw new VarContext.EvalException("subdrive(): Not enough hierarchy information");
-        Nodable no = (Nodable)info;                                 // this inst has LE.subdrive(...) on it
+        Nodable no = (Nodable)info;                                 // this instance has LE.subdrive(...) on it
         if (no == null)
             throw new VarContext.EvalException("subdrive(): Not enough hierarchy");
 
@@ -197,7 +200,7 @@ public class LETool extends Tool {
             // is going to give us icon cell, not equivalent schematic cell
             // We need to re-evaluate networks to get equivalent schematic cell
             NodeInst ni = (NodeInst)no;
-            Cell parent = no.getParent();                               // Cell in which inst which has LE.subdrive is
+            Cell parent = no.getParent();                               // Cell in which instance which has LE.subdrive is
             if (parent == null)
                 throw new VarContext.EvalException("subdrive(): null parent");
 			int arrayIndex = 0;                                         // just use first index
@@ -226,7 +229,6 @@ public class LETool extends Tool {
         }
 
         Variable var = ni.getParameterOrVariable(parName);                          // find variable on nodeinst
-        //if (var == null) return "subdrive(): no variable of name "+parName.replaceFirst("ATTR_", "");
         if (var == null)
             throw new VarContext.EvalException(parName.replaceFirst("ATTR_", "")+" not found");
         return context.push(no).evalVarRecurse(var, ni);                       // evaluate variable and return it
@@ -235,7 +237,7 @@ public class LETool extends Tool {
     /**
      * Attempt to get old style LEDRIVE off of <CODE>no</CODE> based
      * on the VarContext <CODE>context</CODE>.
-     * Attemps to compensate for the situation when the user
+     * Attempts to compensate for the situation when the user
      * had added extra hierarchy to the top of the hierarchy.
      * It cannot compensate for the user has less hierarchy than
      * is required to create the correct Variable name.
@@ -339,26 +341,26 @@ public class LETool extends Tool {
         return null;
     }
 
-    /**
-     * Makes a string denoting hierarchy path.
-     * @param context var context of node
-     * @return  a string denoting hierarchical path of node
-     */
-    private static String makeDriveStr(VarContext context) {
-        return "LEDRIVE_" + context.getInstPath(".");
-    }
+//    /**
+//     * Makes a string denoting hierarchy path.
+//     * @param context var context of node
+//     * @return  a string denoting hierarchical path of node
+//     */
+//    private static String makeDriveStr(VarContext context) {
+//        return "LEDRIVE_" + context.getInstPath(".");
+//    }
 
-    /**
-     * Makes a string denoting hierarchy path.
-     * This is the old version compatible with Java Electric.
-     * @param context var context of node
-     * @return  a string denoting hierarchical path of node
-     */
-    private static String makeDriveStrOLD(VarContext context) {
-        String s = "LEDRIVE_"+makeDriveStrOLDRecurse(context)+";0;S";
-        //System.out.println("name is "+s);
-        return s;
-    }
+//    /**
+//     * Makes a string denoting hierarchy path.
+//     * This is the old version compatible with Java Electric.
+//     * @param context var context of node
+//     * @return  a string denoting hierarchical path of node
+//     */
+//    private static String makeDriveStrOLD(VarContext context) {
+//        String s = "LEDRIVE_"+makeDriveStrOLDRecurse(context)+";0;S";
+//        //System.out.println("name is "+s);
+//        return s;
+//    }
 
     private static String makeDriveStrOLDRecurse(VarContext context) {
         if (context == VarContext.globalContext) return "";
@@ -454,7 +456,6 @@ public class LETool extends Tool {
      */
     public static class AnalyzeCell extends Job
     {
-        /** progress */                         private String progress;
         /** cell to analyze */                  private Cell cell;
         /** var context */                      private VarContext context;
         /** algorithm type */                   private LESizer.Alg algorithm;
@@ -463,7 +464,6 @@ public class LETool extends Tool {
 
         public AnalyzeCell(LESizer.Alg algorithm, Cell cell, VarContext context, boolean newAlg) {
             super("Analyze "+cell, tool, Job.Type.CLIENT_EXAMINE, null, cell, Job.Priority.USER);
-            progress = null;
             this.algorithm = algorithm;
             this.cell = cell;
             this.context = context;
@@ -471,6 +471,7 @@ public class LETool extends Tool {
         }
 
         public boolean doIt() throws JobException {
+        	timer.start();
             // delete last job, if any
             if (lastLEJobExecuted != null)
                 lastLEJobExecuted.remove();
@@ -479,15 +480,6 @@ public class LETool extends Tool {
             setProgress("building equations");
             System.out.print("Building equations...");
 
-            // sleep for testing purposes only, remove later
-//            try {
-//                boolean donesleeping = false;
-//                while(!donesleeping) {
-//                    Thread.sleep(1);
-//                    donesleeping = true;
-//                }
-//            } catch (InterruptedException e) {}
-
             // get sizer and netlister
             Technology layoutTech = cell.getTechnology();
             if (layoutTech == Schematics.tech())
@@ -501,7 +493,7 @@ public class LETool extends Tool {
 
             // calculate statistics
             timer.end();            
-            System.out.println("done ("+timer+")");
+            System.out.println("Done (took " + timer + ")");
 
             // if user aborted, return, and do not run sizer
             if (checkAbort(null)) {
@@ -553,7 +545,7 @@ public class LETool extends Tool {
        }
 
         /**
-         * Check if we are scheduled to abort. If so, print msg if non null
+         * Check if we are scheduled to abort. If so, print message if non null
          * and return true.
          * @param msg message to print if we are aborted
          * @return true on abort, false otherwise
@@ -637,6 +629,7 @@ public class LETool extends Tool {
                     // as top level cell parameter
                     String varName = varNames.get(i);
                     cell.newVar(varName, f, ep);
+                    
                 }
             }
 
@@ -708,7 +701,6 @@ public class LETool extends Tool {
 	                    String p = TextUtils.formatDouble(100*percentdiff, 1);
 	                    String ideal = TextUtils.formatDouble(targetsize, 2);
 	                    String used = TextUtils.formatDouble(chosensize, 2);
-	                    //String strcontext = (context == VarContext.globalContext ? "topcell" : context.getInstPath("."));
 	                    String strcontext = context.push(ni).getInstPath(".");
 	                    System.out.println("  "+p+"%: "+ideal+" (ideal) vs "+used+" (used); for "+strcontext);
 	                }
@@ -782,7 +774,7 @@ public class LETool extends Tool {
          * recursive.
          * @param cell current cell
          * @param context current context
-         * @param ep EditingPreferences ep
+         * @param ep EditingPreferences
          */
         public void registerCellsToUniquify(Cell cell, VarContext context, EditingPreferences ep) {
             Netlist netlist = cell.getNetlist();
@@ -1020,11 +1012,11 @@ public class LETool extends Tool {
     }
 
     public static void clearStoredSizesJob(NodeInst ni) {
-        ClearStoredSizes job = new ClearStoredSizes(ni);
+        new ClearStoredSizes(ni);
     }
 
     public static void clearStoredSizesJob(Library lib) {
-        ClearStoredSizesLibrary job = new ClearStoredSizesLibrary(lib);
+        new ClearStoredSizesLibrary(lib);
     }
 
     /**
@@ -1085,7 +1077,7 @@ public class LETool extends Tool {
         }
     }
 
-    // delete all vars that start with "LEDRIVE_"
+    // delete all variables that start with "LEDRIVE_"
     private static void clearStoredSizes(NodeInst ni) {
         for (Iterator<Variable> it = ni.getVariables(); it.hasNext(); ) {
             Variable var = (Variable)it.next();
@@ -1096,56 +1088,8 @@ public class LETool extends Tool {
         }
     }
 
-/*
-    protected static void addSettings(NetlisterConstants constants, Cell cell) {
-        // find LESETTINGS cell
-        Cell settings = null;
-        for (Iterator libIt = Library.getLibraries(); libIt.hasNext(); ) {
-            Library lib = (Library)libIt.next();
-            for (Iterator it = lib.getCells(); it.hasNext(); ) {
-                Cell c = (Cell)it.next();
-                if (c.getVar("ATTR_LESETTINGS") != null) {
-                    settings = c;
-                    break;
-                }
-            }
-        }
-        if (settings == null) {
-            System.out.println("Did not find LESETTINGS Cell, not saving settings to "+cell.describe());
-            return;
-        }
-        Rectangle2D bounds = cell.getBounds();
-        int x = (int)bounds.getMaxX();
-        int y = (int)bounds.getMinY();
-        NodeInst ni = NodeInst.makeInstance(settings, new Point2D.Double(x,y), settings.getDefWidth(),
-                settings.getDefHeight(), cell);
-        if (ni == null) {
-            System.out.println("Could not create LESETTINGS instance, not saving settings to "+cell.describe());
-            return;
-        }
-        Variable var;
-        if ((var = ni.getVar("ATTR_su")) != null) ni.updateVar(var.getKey(), new Float(constants.su));
-        if ((var = ni.getVar("ATTR_wire_ratio")) != null) ni.updateVar(var.getKey(), new Float(constants.wireRatio));
-        if ((var = ni.getVar("ATTR_epsilon")) != null) ni.updateVar(var.getKey(), new Float(constants.epsilon));
-        if ((var = ni.getVar("ATTR_max_iter")) != null) ni.updateVar(var.getKey(), new Integer(constants.maxIterations));
-        if ((var = ni.getVar("ATTR_gate_cap")) != null) ni.updateVar(var.getKey(), new Float(constants.gateCap));
-        if ((var = ni.getVar("ATTR_alpha")) != null) ni.updateVar(var.getKey(), new Float(constants.alpha));
-        if ((var = ni.getVar("ATTR_keeper_ratio")) != null) ni.updateVar(var.getKey(), new Float(constants.keeperRatio));
-    }
-*/
-
 	/************************** PROJECT PREFERENCES *************************/
 
-//	// preferences; tech-dependent values handled by technology and
-    // project preferences
-    private static double DEFAULT_GLOBALFANOUT = 4.7;
-    private static double DEFAULT_EPSILON      = 0.001;
-    private static int    DEFAULT_MAXITER      = 30;
-//    private static double DEFAULT_GATECAP      = 0.4;
-//    private static double DEFAULT_WIRERATIO    = 0.16;
-//    private static double DEFAULT_DIFFALPHA    = 0.7;
-    private static double DEFAULT_KEEPERRATIO  = 0.1;
-
 	/**
 	 * Method to tell whether to use local settings for Logical Effort.
 	 * The default is true.
@@ -1158,32 +1102,6 @@ public class LETool extends Tool {
 	 */
 	public static Setting getUseLocalSettingsSetting() { return ToolSettings.getUseLocalSettingsSetting(); }
 
-//	private static Pref cacheHighlightComponents = Pref.makeBooleanPref("HighlightComponents", LETool.tool.prefs, false);
-//	/**
-//	 * Method to tell whether to highlight components in Logical Effort.
-//	 * The default is false.
-//	 * @return true to highlight components in Logical Effort
-//	 */
-//	public static boolean isHighlightComponents() { return cacheHighlightComponents.getBoolean(); }
-//	/**
-//	 * Method to set whether to highlight components in Logical Effort
-//	 * @param on whether to highlight components in Logical Effort
-//	 */
-//	public static void setHighlightComponents(boolean on) { cacheHighlightComponents.setBoolean(on); }
-
-//	private static Pref cacheShowIntermediateCapacitances = Pref.makeBooleanPref("ShowIntermediateCapacitances", LETool.tool.prefs, false);
-//	/**
-//	 * Method to tell whether to highlight intermediate capacitances in Logical Effort.
-//	 * The default is false.
-//	 * @return true to highlight intermediate capacitances in Logical Effort
-//	 */
-//	public static boolean isShowIntermediateCapacitances() { return cacheShowIntermediateCapacitances.getBoolean(); }
-//	/**
-//	 * Method to set whether to highlight intermediate capacitances in Logical Effort
-//	 * @param on whether to highlight intermediate capacitances in Logical Effort
-//	 */
-//	public static void setShowIntermediateCapacitances(boolean on) { cacheShowIntermediateCapacitances.setBoolean(on); }
-
 	/**
 	 * Method to get the Global Fanout for Logical Effort.
 	 * The default is DEFAULT_GLOBALFANOUT.
@@ -1220,45 +1138,6 @@ public class LETool extends Tool {
 	 */
 	public static Setting getMaxIterationsSetting() { return ToolSettings.getMaxIterationsSetting(); }
 
-//	private static Pref cacheGateCapacitance = Pref.makeDoublePref("GateCapfFPerLambda", LETool.tool.prefs, DEFAULT_GATECAP);
-//	/**
-//	 * Method to get the Gate Capacitance for Logical Effort.
-//	 * The default is DEFAULT_GATECAP.
-//	 * @return the Gate Capacitance for Logical Effort.
-//	 */
-//	public static double getGateCapacitance() { return cacheGateCapacitance.getDouble(); }
-//	/**
-//	 * Method to set the Gate Capacitance for Logical Effort.
-//	 * @param gc the Gate Capacitance for Logical Effort.
-//	 */
-//	public static void setGateCapacitance(double gc) { cacheGateCapacitance.setDouble(gc); }
-
-//	private static Pref cacheWireRatio = Pref.makeDoublePref("WireRatio", LETool.tool.prefs, DEFAULT_WIRERATIO);
-//	/**
-//	 * Method to get the wire capacitance ratio for Logical Effort.
-//	 * The default is DEFAULT_WIRERATIO.
-//	 * @return the wire capacitance ratio for Logical Effort.
-//	 */
-//	public static double getWireRatio() { return cacheWireRatio.getDouble(); }
-//	/**
-//	 * Method to set the wire capacitance ratio for Logical Effort.
-//	 * @param wr the wire capacitance ratio for Logical Effort.
-//	 */
-//	public static void setWireRatio(double wr) { cacheWireRatio.setDouble(wr); }
-
-//	private static Pref cacheDiffAlpha = Pref.makeDoublePref("DiffusionAlpha", LETool.tool.prefs, DEFAULT_DIFFALPHA);
-//	/**
-//	 * Method to get the diffusion to gate capacitance ratio for Logical Effort.
-//	 * The default is DEFAULT_DIFFALPHA.
-//	 * @return the diffusion to gate capacitance ratio for Logical Effort.
-//	 */
-//	public static double getDiffAlpha() { return cacheDiffAlpha.getDouble(); }
-//	/**
-//	 * Method to set the diffusion to gate capacitance ratio for Logical Effort.
-//	 * @param da the diffusion to gate capacitance ratio for Logical Effort.
-//	 */
-//	public static void setDiffAlpha(double da) { cacheDiffAlpha.setDouble(da); }
-
 	/**
 	 * Method to get the keeper size ratio for Logical Effort.
 	 * The default is DEFAULT_KEEPERRATIO.
