@@ -6,7 +6,7 @@
  * Routing tool: Sea of Gates control
  * Written by: Steven M. Rubin
  *
- * Copyright (c) 2007, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, Static Free Software. All rights reserved.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,6 +41,7 @@ import com.sun.electric.technology.Technology;
 import com.sun.electric.technology.technologies.Generic;
 import com.sun.electric.tool.Job;
 import com.sun.electric.tool.routing.Routing.SoGContactsStrategy;
+import com.sun.electric.tool.routing.Routing.SoGNetOrder;
 import com.sun.electric.tool.routing.seaOfGates.SeaOfGatesEngine;
 import com.sun.electric.tool.routing.seaOfGates.SeaOfGatesEngineFactory.SeaOfGatesEngineType;
 import com.sun.electric.tool.routing.seaOfGates.SeaOfGatesHandlers;
@@ -80,7 +81,7 @@ public class SeaOfGates
      * Presumes that it is inside of a Job.
      * @param justSubCells true to run the router on all sub-cells of the current cell
      * (but not on the current cell itself).
-     * 
+     *
      *                  standard entry to routing is seaOfGatesRoute(false)
      */
     public static void seaOfGatesRoute(boolean justSubCells)
@@ -121,8 +122,8 @@ public class SeaOfGates
     /**
      * Method to run Sea-of-Gates routing on the current cell, using a specified routing engine type.
      * Presumes that it is inside of a Job.
-     * 
-     *                  standard entry to routing is seaOfGatesRoute(SeaOfGatesEngineType.defaultVersion)
+     *
+     * standard entry to routing is seaOfGatesRoute(SeaOfGatesEngineType.defaultVersion)
      */
     public static void seaOfGatesRoute(SeaOfGatesEngineType version)
     {
@@ -265,11 +266,12 @@ public class SeaOfGates
         public double maxArcWidth;
         public int complexityLimit, rerunComplexityLimit;
         public int maxDistance;
-        public boolean useGlobalRouter, reRunFailedRoutes, enableSpineRouting;
+        public boolean useGlobalRouter, reRunFailedRoutes, enableSpineRouting, disableAdvancedSpineRouting;
         public int forcedNumberOfThreads;
         public String resultCellName;
         public ElapseTimer theTimer;
         public PrintStream qualityPrintStream;
+        public SoGNetOrder netOrder;
 
         public SeaOfGatesOptions()
         {
@@ -283,12 +285,14 @@ public class SeaOfGates
             useGlobalRouter = false;
             reRunFailedRoutes = false;
             enableSpineRouting = false;
+            disableAdvancedSpineRouting = false;
             forcedNumberOfThreads = 0;
             resultCellName = null;
             qualityPrintStream = null;
             maxDistance = 2; // original default
+            netOrder = SoGNetOrder.SOGNETORDERORIGINAL; // original strategy
         }
-        
+
         public void getOptionsFromPreferences(boolean factory)
         {
         	if (factory)
@@ -305,6 +309,7 @@ public class SeaOfGates
 	            enableSpineRouting = Routing.isFactorySeaOfGatesEnableSpineRouting();
 	            forcedNumberOfThreads = Routing.getFactorySeaOfGatesForcedProcessorCount();
 	            maxDistance = Routing.getFactorySeaOfGatesMaxDistance();
+	            netOrder = Routing.getFactorySeaOfGatesNetOrder();
         	}
         	else
         	{
@@ -320,8 +325,31 @@ public class SeaOfGates
 	            enableSpineRouting = Routing.isSeaOfGatesEnableSpineRouting();
 	            forcedNumberOfThreads = Routing.getSeaOfGatesForcedProcessorCount();
 	            maxDistance = Routing.getSeaOfGatesMaxDistance();
+	            netOrder = Routing.getSeaOfGatesNetOrder();
         	}
         }
+
+        /**
+         * Method to print active preferences for debugging purposes
+         */
+        @Override
+        public String toString()
+        {
+        	return "SoG Preferences = (" +
+        			"useParallelFromToRoutes=" + useParallelFromToRoutes + ", " +
+        			"useParallelRoutes=" + useParallelRoutes + ", " +
+        			"runOnConnectedRoutes=" + runOnConnectedRoutes + ",\n " +
+        			"contactPlacementAction=" + contactPlacementAction + ", " +
+        			"maxArcWidth=" + maxArcWidth + ", " +
+        			"complexityLimit=" + complexityLimit + ",\n " +
+        			"rerunComplexityLimit=" + rerunComplexityLimit + ", " +
+        			"useGlobalRouter=" + useGlobalRouter + ", " +
+        			"reRunFailedRoutes=" + reRunFailedRoutes + ",\n " +
+        			"forcedNumberOfThreads=" + forcedNumberOfThreads + ", " +
+        			"maxDistance=" + maxDistance + ", " +
+        			"netOrder=" + netOrder.getValue() +
+        			")";
+        }
     }
 
     public static class SeaOfGatesArcProperties implements Serializable
@@ -344,17 +372,17 @@ public class SeaOfGates
     	public void setWidthOverride(Double w) { overrideWidth = w; }
     	public Double getWidthOverride() { return overrideWidth; }
 
-    	public void setSpacingOverride(Double s, int axis) 
-    	{ 
+    	public void setSpacingOverride(Double s, int axis)
+    	{
     		if (axis == 0) overrideSpacingX = s;
     		else overrideSpacingY = s;
     	}
-    	
-    	public Double getSpacingOverride(int axis) 
-    	{ 
-    		return (axis == 0) ? overrideSpacingX : overrideSpacingY; 
+
+    	public Double getSpacingOverride(int axis)
+    	{
+    		return (axis == 0) ? overrideSpacingX : overrideSpacingY;
     	}
-    	
+
     	public void setWidthOf2X(Double w) { widthOf2X = w; }
     	public Double getWidthOf2X() { return widthOf2X; }
 
@@ -483,12 +511,12 @@ public class SeaOfGates
 					{
 						canPlaceContactDownToAvoidedLayer = false;
 						continue;
-					}		
+					}
 					if (parts[0].equalsIgnoreCase("PreventContactUpToAvoidedLayer"))
 					{
 						canPlaceContactUpToAvoidedLayer = false;
 						continue;
-					}					
+					}
 					if (parts[0].equalsIgnoreCase("NoContactRotation"))
 					{
 						canRotateContacts = false;
@@ -639,11 +667,6 @@ public class SeaOfGates
 						}
 						continue;
 					}
-
-//					if (parts[0].toLowerCase().startsWith("ArcSpacingOverride") && parts.length >= 3)
-//					{
-//						int axis = (parts[0].substring(parts[0].length()-1).equals("X")) ? 0 : 1;
-//					}
 					if (parts[0].equalsIgnoreCase("NetworkOverride") && parts.length >= 2)
 					{
 						String netName = parts[1];
@@ -673,11 +696,11 @@ public class SeaOfGates
 							continue;
 						}
 						continue;
-					}					
+					}
 				}
 			}
 		}
-		
+
 	    /**
 	     * Method to import the control file for routing.
 	     * @param fileName the name of the control file.
@@ -711,8 +734,6 @@ public class SeaOfGates
 	    			if (buf == null) break;
 	    			if (buf.length() == 0 || buf.startsWith(";")) continue;
 
-					//String[] parts = buf.split(" ");
-	    			// Using StringTokenizer to deal with white spaces
 	    			List<String> slist = new ArrayList<String>();
 	    			StringTokenizer p = new StringTokenizer(buf, " ", false);
 	                while (p.hasMoreTokens())
@@ -738,7 +759,7 @@ public class SeaOfGates
 
 					if (parts[0].equalsIgnoreCase("RoutingBoundsLayer"))
 					{
-						// define routing bound layer 
+						// define routing bound layer
 						assert(parts.length > 1);
 						routingBoundsLayerName = parts[1];
 						continue;
@@ -789,6 +810,11 @@ public class SeaOfGates
 						assert(parts.length > 1);
 						// just getting first element
 						List<ArcProto> apList = getArcProtoList(parts[1], lineReader.getLineNumber(), curTech, extraLayers);
+						if (apList == null)
+						{
+							System.out.println("ERROR parsing line " + lineReader.getLineNumber() + " of file");
+							continue;
+						}
 						for (ArcProto ap: apList)
 						{
 							int level = ap.getFunction().getLevel();
@@ -803,6 +829,11 @@ public class SeaOfGates
 						assert(parts.length > 1);
 						// just getting first element
 						List<ArcProto> apList = getArcProtoList(parts[1], lineReader.getLineNumber(), curTech, extraLayers);
+						if (apList == null)
+						{
+							System.out.println("ERROR parsing line " + lineReader.getLineNumber() + " of file");
+							continue;
+						}
 						for (ArcProto ap: apList)
 						{
 							int level = ap.getFunction().getLevel();
@@ -817,7 +848,7 @@ public class SeaOfGates
 						for(int i=1; i<parts.length; i++)
 						{
 							List<ArcProto> apList = getArcProtoList(parts[i], lineReader.getLineNumber(), curTech, extraLayers);
-							allowed.addAll(apList);
+							if (apList != null) allowed.addAll(apList);
 						}
 						for(Iterator<ArcProto> it = curTech.getArcs(); it.hasNext(); )
 						{
@@ -833,7 +864,7 @@ public class SeaOfGates
 						for(int i=1; i<parts.length; i++)
 						{
 							List<ArcProto> apList = getArcProtoList(parts[i], lineReader.getLineNumber(), curTech, extraLayers);
-							allowed.addAll(apList);
+							if (apList != null) allowed.addAll(apList);
 						}
 						for(ArcProto ap : allowed)
 							setTaperOnly(ap, true);
@@ -844,6 +875,8 @@ public class SeaOfGates
 					{
 						tmpList.clear();
 						List<ArcProto> apList = getArcProtoList(parts[1], lineReader.getLineNumber(), curTech, extraLayers);
+						if (apList == null) continue;
+
 						// Checking there is previous definition of those layers
 						for (ArcProto arc : apList)
 						{
@@ -861,7 +894,7 @@ public class SeaOfGates
 							double spacingX = TextUtils.atof(parts[3]);
 							spacingX = TextUtils.convertFromDistance(spacingX, curTech, UnitScale.MICRO);
 							setDefaultSpacingOverride(arc, spacingX, 0);
-							
+
 							double spacingY = TextUtils.atof(parts[4]);
 							spacingY = TextUtils.convertFromDistance(spacingY, curTech, UnitScale.MICRO);
 							setDefaultSpacingOverride(arc, spacingY, 1);
@@ -880,6 +913,7 @@ public class SeaOfGates
 					{
 						tmpList.clear();
 						List<ArcProto> apList = getArcProtoList(parts[1], lineReader.getLineNumber(), curTech, extraLayers);
+						if (apList == null) continue;
 						for (ArcProto arc : apList)
 						{
 							double taperLength = TextUtils.atof(parts[2]);
@@ -898,6 +932,7 @@ public class SeaOfGates
 							continue;
 						}
 						List<ArcProto> apList = getArcProtoList(parts[1], lineReader.getLineNumber(), curTech, extraLayers);
+						if (apList == null) continue;
 						int maskNum = SeaOfGatesTrack.getSpecificMaskNumber(parts[1]);
 						for (ArcProto ap : apList)
 						{
@@ -915,6 +950,7 @@ public class SeaOfGates
 							continue;
 						}
 						List<ArcProto> apList = getArcProtoList(parts[1], lineReader.getLineNumber(), curTech, extraLayers);
+						if (apList == null) continue;
 						if (apList.size() < 1)
 						{
 							System.out.println("ERROR: unknown arc on line " + lineReader.getLineNumber());
@@ -943,6 +979,7 @@ public class SeaOfGates
 						}
 
 						List<ArcProto> apList = getArcProtoList(parts[1], lineReader.getLineNumber(), curTech, extraLayers);
+						if (apList == null) continue;
 						int maskNum = SeaOfGatesTrack.getSpecificMaskNumber(parts[1]);
 						for (ArcProto ap : apList)
 						{
@@ -1050,6 +1087,7 @@ public class SeaOfGates
 						for(int i=1; i<parts.length; i++)
 						{
 							List<ArcProto> apList = getArcProtoList(parts[i], lineReader.getLineNumber(), curTech, extraLayers);
+							if (apList == null) continue;
 							for (ArcProto ap : apList)
 								layersToOverride.add(ap);
 						}
@@ -1061,6 +1099,7 @@ public class SeaOfGates
 						for(int i=1; i<parts.length; i += 2)
 						{
 							List<ArcProto> apList = getArcProtoList(parts[i], lineReader.getLineNumber(), curTech, extraLayers);
+							if (apList == null) continue;
 							for (ArcProto ap : apList)
 							{
 								double v = TextUtils.atof(parts[i+1]);
@@ -1076,6 +1115,7 @@ public class SeaOfGates
 						for(int i=1; i<parts.length; i += 2)
 						{
 							List<ArcProto> apList = getArcProtoList(parts[i], lineReader.getLineNumber(), curTech, extraLayers);
+							if (apList == null) continue;
 							for (ArcProto ap : apList)
 							{
 								double v = TextUtils.atof(parts[i+1]);
@@ -1313,7 +1353,7 @@ public class SeaOfGates
 			ArcProto ap = tech.findArcProto(layerName);
 			return ap;
 		}
-		
+
 		/**
 		 * Function to determine the direction of the primary spacing value
 		 * @param ap ArcProto representing the metal arc
@@ -1595,7 +1635,7 @@ public class SeaOfGates
 		 * @return true if the layer is favored.
 		 */
 		public boolean isFavored(ArcProto ap) { return favoredArcs.contains(ap); }
-		
+
 		/**
 		 * Method to set whether a given layer is available only for tapers.
 		 * Tapers are ends of a route that take the width of the endpoint node.
@@ -1661,7 +1701,7 @@ public class SeaOfGates
 		}
 
 		/**
-		 * Method to tell whether a given layer should use a different spacing along a given axis 
+		 * Method to tell whether a given layer should use a different spacing along a given axis
 		 * when routing the Cell. 0 for X, 1 for Y.
 		 * @param ap the ArcProto of the layer in question.
 		 * @return the arc spacing to use (null to use the arc's default).
@@ -1674,7 +1714,7 @@ public class SeaOfGates
 		}
 
 		/**
-		 * Method to set whether a given layer should use a different spacing along a given axis 
+		 * Method to set whether a given layer should use a different spacing along a given axis
 		 * when routing the Cell. 0 for X, 1 for Y.
 		 * @param ap the ArcProto of the layer in question.
 		 * @param s the new spacing to use (null to use the default).
@@ -1870,7 +1910,6 @@ public class SeaOfGates
 		 * @param net the name of the network.
 		 * @param ap the ArcProto being routed on the network.
 		 * @param spacing the arc spacing to use (null to remove the override).
-		 * @param axis direction for the spacing value
 		 */
 		public void setSpacingOverrideForArcOnNet(String net, ArcProto ap, Double spacing)
 		{
@@ -1946,7 +1985,7 @@ public class SeaOfGates
 		 * @param s String representing the regular expression
 		 */
 		public void setAcceptOnly2XPrimitives(String s) {acceptOnly2XPrimitives = s; }
-		
+
 		/**
 		 * Method to return the name of the layer to be used to limit routing bounds.
 		 * @return the name of the layer to be used to limit routing bounds (null if no limiting being done).
