@@ -4,7 +4,7 @@
  *
  * File: SeaOfGatesCellBuilder.java
  *
- * Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, Static Free Software. All rights reserved.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,7 +33,6 @@ import com.sun.electric.database.ImmutableNodeInst;
 import com.sun.electric.database.Snapshot;
 import com.sun.electric.database.geometry.EGraphics;
 import com.sun.electric.database.geometry.EPoint;
-import com.sun.electric.database.geometry.ERectangle;
 import com.sun.electric.database.geometry.Poly;
 import com.sun.electric.database.hierarchy.View;
 import com.sun.electric.database.id.ArcProtoId;
@@ -45,22 +44,18 @@ import com.sun.electric.database.id.PrimitiveNodeId;
 import com.sun.electric.database.prototype.PortCharacteristic;
 import com.sun.electric.database.text.CellName;
 import com.sun.electric.database.text.Name;
-import com.sun.electric.database.text.TextUtils;
 import com.sun.electric.database.variable.TextDescriptor;
 import com.sun.electric.technology.AbstractShapeBuilder;
-import com.sun.electric.technology.EdgeH;
-import com.sun.electric.technology.EdgeV;
 import com.sun.electric.technology.Layer;
 import com.sun.electric.technology.PrimitiveNode;
 import com.sun.electric.technology.PrimitivePort;
 import com.sun.electric.technology.TechPool;
-import com.sun.electric.technology.Technology;
 import com.sun.electric.technology.technologies.Generic;
 import com.sun.electric.tool.Job;
 import com.sun.electric.tool.Tool;
 import com.sun.electric.tool.routing.Routing.SoGContactsStrategy;
-import com.sun.electric.util.math.FixpCoord;
-import com.sun.electric.util.math.GenMath;
+import com.sun.electric.tool.routing.seaOfGates.SeaOfGatesHandlers.ContactKey;
+import com.sun.electric.tool.routing.seaOfGates.SeaOfGatesHandlers.ContactTemplate;
 import com.sun.electric.util.math.Orientation;
 
 import java.util.ArrayList;
@@ -77,33 +72,34 @@ import java.util.Set;
 import java.util.TreeMap;
 
 /**
- *
- * @author dn146861
+ * Class for creating final circuitry in Sea of Gates router.
+ * @author Dmitry Nadezhin
  */
 class SeaOfGatesCellBuilder {
 
 	private static final boolean FINDEXISTINGCELLS = true;
 
 	private final EditingPreferences ep;
-    // oldSnapshot
+
+	// oldSnapshot
     private final Snapshot oldSnapshot;
     private final Tool oldTool;
     private final Environment oldEnvironment;
     private final CellTree oldCellTree;
     private final CellBackup oldCellBackup;
-//    private final CellTree[] oldCellTrees;
     private final TechPool oldTechPool;
     private final CellRevision oldCellRevision;
     private final ImmutableCell oldCell;
     final CellId cellId;
     final String resultCellName;
     final SoGContactsStrategy contactPlacementAction;
+
     // curSnapshot
     private Snapshot curSnapshot;
     private final List<CellBackup> curCellBackups;
-//    private CellTree curCellTree;
     private CellBackup curCellBackup;
     private Map<ContactKey,ContactTemplate> contactTemplates = new HashMap<ContactKey,ContactTemplate>();
+
     // nodes
     private final BitSet curNodes = new BitSet();
     private int curNodesCount;
@@ -112,13 +108,14 @@ class SeaOfGatesCellBuilder {
     private final IdentityHashMap<PrimitiveNodeId, MaxNodeSuffix> maxNodeSuffixes = new IdentityHashMap<PrimitiveNodeId, MaxNodeSuffix>();
     private final IdentityHashMap<SeaOfGatesEngine.RouteNode, ImmutableNodeInst> addedNodesToNodeInst = new IdentityHashMap<SeaOfGatesEngine.RouteNode, ImmutableNodeInst>();
     private final IdentityHashMap<SeaOfGatesEngine.RouteNode, PortProtoId> addedNodesToPortProtoId = new IdentityHashMap<SeaOfGatesEngine.RouteNode, PortProtoId>();
+
     // arcs
     private final List<ImmutableArcInst> curArcs = new ArrayList<ImmutableArcInst>();
     private int maxArcSuffix;
 
     // result CellBackup
     private final MyShapeBuilder resultShapeBuilder;
-    
+
     SeaOfGatesCellBuilder(Snapshot snapshot, CellId cellId, String resultCellName, SoGContactsStrategy contactPlacementAction, EditingPreferences ep) {
         this.ep = ep;
 
@@ -131,12 +128,10 @@ class SeaOfGatesCellBuilder {
         oldEnvironment = oldSnapshot.environment;
         oldCellTree = snapshot.getCellTree(cellId);
         oldCellBackup = oldCellTree.top;
-//        oldCellTrees = oldCellTree.getSubTrees();
         oldTechPool = oldCellTree.techPool;
         oldCellRevision = oldCellBackup.cellRevision;
         oldCell = oldCellRevision.d;
 
-
         curCellBackups = new ArrayList<CellBackup>();
         if (resultCellName != null) {
             CellId resultCellId = cellId.libId.newCellId(CellName.newName(resultCellName, View.LAYOUT, 1));
@@ -152,7 +147,7 @@ class SeaOfGatesCellBuilder {
                 int cellIndex = cellTree.top.cellRevision.d.cellId.cellIndex;
                 while (curCellBackups.size() <= cellIndex) {
                     curCellBackups.add(null);
-                } 
+                }
                 curCellBackups.set(cellIndex, cellTree.top);
             }
         }
@@ -207,7 +202,6 @@ class SeaOfGatesCellBuilder {
         }
 
         curSnapshot = oldSnapshot;
-//        curCellTree = oldCellTree;
         curCellBackup = oldCellBackup;
     }
 
@@ -220,10 +214,10 @@ class SeaOfGatesCellBuilder {
 	 */
 	private ContactTemplate getTemplateForContact(PrimitiveNode pNp, EPoint size, int techBits)
     {
-        ContactKey contactKey = new ContactKey(pNp, size, techBits);
+		SeaOfGatesHandlers.ContactKey contactKey = new SeaOfGatesHandlers.ContactKey(pNp, size, techBits);
         return contactTemplates.get(contactKey);
     }
-    
+
 	/**
 	 * Method to create a CellId to describe a given primitive node, when encapsulated
 	 * @param pNp the contact to package.
@@ -234,27 +228,19 @@ class SeaOfGatesCellBuilder {
 	private void makeTemplateForContact(PrimitiveNode pNp, EPoint size, int techBits)
     {
         assert pNp.getFunction().isContact();
-        ContactKey contactKey = new ContactKey(pNp, size, techBits);
+        SeaOfGatesHandlers.ContactKey contactKey = new SeaOfGatesHandlers.ContactKey(pNp, size, techBits);
 
         Name nodeName = pNp.getPrimitiveFunction(techBits).getBasename().findSuffixed(0);
-        nodeName = (contactPlacementAction == SoGContactsStrategy.SOGCONTACTSUSEEXISTINGSUBCELLS) ? 
+        nodeName = (contactPlacementAction == SoGContactsStrategy.SOGCONTACTSUSEEXISTINGSUBCELLS) ?
         		pNp.getPrimitiveFunction(techBits).getBasename().findSuffixed(0) : Name.findName(contactKey.toString());
         CellBackup contactCellBackup = null;
         Orientation contactTemplateOrientation = Orientation.IDENT;
         if (FINDEXISTINGCELLS && contactPlacementAction != SoGContactsStrategy.SOGCONTACTSFORCESUBCELLS)
         {
-//System.out.println("LOOKING FOR A MATCH FOR PRIMITIVE "+contactKey);
             ImmutableNodeInst n = ImmutableNodeInst.newInstance(0, pNp.getId(), nodeName, TextDescriptor.EMPTY, Orientation.IDENT,
                 EPoint.ORIGIN, size, 0, techBits, TextDescriptor.EMPTY);
             MyShapeBuilder conShape = new MyShapeBuilder();
             pNp.genShape(conShape, n);
-//System.out.println("Con shape:");
-//for (ImmutableNodeInst sn: conShape.nodes)
-//{
-//    PrimitiveNode pn = oldTechPool.getPrimitiveNode((PrimitiveNodeId) sn.protoId);
-//    assert pn.getFunction() == PrimitiveNode.Function.NODE;
-//    System.out.println("  " + pn + " " + sn.size + " " + sn.anchor);
-//}
             MyShapeBuilder conShapeRot = new MyShapeBuilder();
             pNp.genShape(conShapeRot, n.withOrient(Orientation.R));
             for (CellBackup cellBackup: curCellBackups)
@@ -263,20 +249,18 @@ class SeaOfGatesCellBuilder {
                 if (cellBackup.cellRevision.d.cellId.libId != cellId.libId) continue;
                 if (doesCellMatch(cellBackup, conShape))
                 {
-//System.out.println("    FOUND CELL "+cellBackup.cellRevision.d.cellId+" THAT MATCHES PRIMITIVE "+contactKey);
                     contactCellBackup = cellBackup;
                     break;
                 }
                 if (doesCellMatch(cellBackup, conShapeRot))
                 {
-//System.out.println("    FOUND CELL "+cellBackup.cellRevision.d.cellId+" THAT (WHEN ROTATED) MATCHES PRIMITIVE "+contactKey);
                     contactCellBackup = cellBackup;
                     contactTemplateOrientation = Orientation.R;
                     break;
                 }
             }
         }
-        
+
         CellId contactCellId;
         if (contactCellBackup != null)
         {
@@ -286,16 +270,15 @@ class SeaOfGatesCellBuilder {
             {
                 // Contact cell has exports, Pick the first.
                 ExportId eId = contactCellBackup.cellRevision.exports.iterator().next().exportId;
-                ContactTemplate contactTemplate = new ContactTemplate(contactCellId, contactTemplateOrientation, eId);
+                SeaOfGatesHandlers.ContactTemplate contactTemplate = new SeaOfGatesHandlers.ContactTemplate(contactCellId, contactTemplateOrientation, eId);
                 contactTemplates.put(contactKey, contactTemplate);
                 return;
             }
             // Instantiate universal pin instead of pure layer node
             pNp = Generic.tech().universalPinNode;
-            nodeName = (contactPlacementAction == SoGContactsStrategy.SOGCONTACTSUSEEXISTINGSUBCELLS) ? 
+            nodeName = (contactPlacementAction == SoGContactsStrategy.SOGCONTACTSUSEEXISTINGSUBCELLS) ?
             		pNp.getPrimitiveFunction(techBits).getBasename().findSuffixed(0) : Name.findName(contactKey.toString());
-        }
-        else
+        } else
         {
         	if (contactPlacementAction == SoGContactsStrategy.SOGCONTACTSUSEEXISTINGSUBCELLS) return;
             // Create a new contact cell
@@ -308,17 +291,17 @@ class SeaOfGatesCellBuilder {
             contactCellBackup = CellBackup.newInstance(c, oldTechPool);
             curCellBackups.set(contactCellId.cellIndex, contactCellBackup);
         }
-        
+
         // Create ImmutableNodeInst
         int nodeId = contactCellBackup.cellRevision.getMaxNodeId() + 1;
         TextDescriptor nameTd = ep.getNodeTextDescriptor();
         EPoint anchor = EPoint.ORIGIN;
         int flags = 0;
         TextDescriptor protoTd = ep.getInstanceTextDescriptor();
-        ImmutableNodeInst n = 
+        ImmutableNodeInst n =
             ImmutableNodeInst.newInstance(nodeId, pNp.getId(), nodeName, nameTd, Orientation.IDENT,
                 anchor, size, flags, techBits, protoTd);
-        
+
         // Create ImmuatbleExport
         String portName = "port";
         ExportId exportId = contactCellId.newPortId(portName);
@@ -327,7 +310,7 @@ class SeaOfGatesCellBuilder {
         boolean bodyOnly = false;
         ImmutableExport e = ImmutableExport.newInstance(exportId, Name.findName(portName), portTd,
             nodeId, pNp.getPort(0).getId(), alwaysDrawn, bodyOnly, PortCharacteristic.UNKNOWN);
-        
+
         // Insert node instance into new nodes array
         ImmutableNodeInst[] nodesArray = new ImmutableNodeInst[contactCellBackup.cellRevision.nodes.size() + 1];
         ImmutableNodeInst nodeToInsert = n;
@@ -348,18 +331,18 @@ class SeaOfGatesCellBuilder {
             nodesArray[nodeIndex++] = nodeToInsert;
         }
         assert nodeIndex == nodesArray.length;
-        
+
         // Update CellBackup
         ImmutableExport[] exportsArray = { e };
         contactCellBackup = contactCellBackup.with(contactCellBackup.cellRevision.d,
             nodesArray, null, exportsArray, oldTechPool);
         curCellBackups.set(contactCellId.cellIndex, contactCellBackup);
-        
+
         // save ContactTemplate
-        ContactTemplate contactTemplate = new ContactTemplate(contactCellId, contactTemplateOrientation, exportId);
+        SeaOfGatesHandlers.ContactTemplate contactTemplate = new SeaOfGatesHandlers.ContactTemplate(contactCellId, contactTemplateOrientation, exportId);
         contactTemplates.put(contactKey, contactTemplate);
     }
-    
+
     private boolean doesCellMatch(CellBackup contactCellBackup, MyShapeBuilder sb)
     {
         List<ImmutableNodeInst> shapeNodes = sb.nodes;
@@ -403,7 +386,7 @@ class SeaOfGatesCellBuilder {
 	            PrimitiveNodeId protoId = (PrimitiveNodeId) rn.getProtoId();
 	            PrimitiveNode pNp = oldTechPool.getPrimitiveNode(protoId);
 	            if (!pNp.getFunction().isContact()) continue;
-	            ContactTemplate contactTemplate = getTemplateForContact(pNp, rn.getSize(), rn.getTechBits());
+	            SeaOfGatesHandlers.ContactTemplate contactTemplate = getTemplateForContact(pNp, rn.getSize(), rn.getTechBits());
                 if (contactTemplate == null)
                 {
                     makeTemplateForContact(pNp, rn.getSize(), rn.getTechBits());
@@ -442,7 +425,7 @@ class SeaOfGatesCellBuilder {
             Orientation orient = Orientation.IDENT;
             if (pNp.getFunction().isContact())
             {
-	            ContactTemplate contactTemplate = getTemplateForContact(pNp, rn.getSize(), rn.getTechBits());
+            	SeaOfGatesHandlers.ContactTemplate contactTemplate = getTemplateForContact(pNp, rn.getSize(), rn.getTechBits());
 	            if (contactTemplate != null)
 	            {
 	            	// place cell "c" instead of primitive "protoId"
@@ -480,7 +463,9 @@ class SeaOfGatesCellBuilder {
                 if (protoId != pnId) {
                     resultShapeBuilder.nodes.add(n);
                 } else {
+					int startNodeId = resultShapeBuilder.getNextNodeId();
                     oldTechPool.getPrimitiveNode(pnId).genShape(resultShapeBuilder, n);
+					n = resultShapeBuilder.nodes.get(startNodeId);
                 }
             } else {
               maxSuffix.add(n);
@@ -502,7 +487,6 @@ class SeaOfGatesCellBuilder {
             if (ra.getName() != null) newName = Name.findName(ra.getName());
             if (newName != null && !newName.isTempname())
         	{
-//            	System.out.println("NAMING NETWORK "+newName);
         		name = newName;
             } else
             {
@@ -581,143 +565,18 @@ class SeaOfGatesCellBuilder {
         resolution.clearRoutes();
     }
 
-    private static class ContactKey
-    {
-        final PrimitiveNode pNp;
-        final EPoint size;
-        final int techBits;
-        
-        ContactKey(PrimitiveNode pNp, EPoint size, int techBits)
-        {
-            if (pNp == null || size == null) throw new NullPointerException();
-            this.pNp = pNp;
-            this.size = size;
-            this.techBits = techBits;
-        }
-        
-        @Override
-        public boolean equals(Object o)
-        {
-            if (o instanceof ContactKey)
-            {
-                ContactKey that = (ContactKey) o;
-                return this.pNp.equals(that.pNp) && this.size.equals(that.size) && this.techBits == that.techBits;
-            }
-            return false;
-        }
-
-        @Override
-        public int hashCode()
-        {
-            return 89 * pNp.hashCode() + size.hashCode() + techBits;
-        }
-        
-        @Override
-        public String toString() {
-        	assert(pNp instanceof PrimitiveNode);
-        	
-        	PrimitiveNode pn = (PrimitiveNode)pNp;
-        	double scale = pn.getTechnology().getScale();
-            ERectangle base = pn.getBaseRectangle();
-//            double wid = size.getX() + base.getWidth();
-//            double hei = size.getY() + base.getHeight();
-//            String s = pNp.getName();
-//            if (techBits != 0)
-//                s += "-" + techBits;
-
-            double[] xExt = new double[2];
-            double[] yExt = new double[2];
-            double[] viaSize = new double[2];
-            double[] viaSpacing = new double[2];
-            String[] names = new String[2];
-            int count = 0;
-            
-            for (Technology.NodeLayer node : pn.getNodeLayers())
-            {
-            	Layer l = node.getLayer();
-                
-            	if (l.getFunction().isContact())
-            	{
-            		// via size
-            		viaSize[0] = (FixpCoord.fixpToLambda(node.getMulticutSizeX().getFixp())); // x value
-            		viaSize[1] = (FixpCoord.fixpToLambda(node.getMulticutSizeY().getFixp())); // y value
-            		// via spacing
-            		// @TODO assuming 1x1 cuts
-            		viaSpacing[0] = (FixpCoord.fixpToLambda(node.getMulticutSep1D().getFixp())); // x value
-            		viaSpacing[1] = (FixpCoord.fixpToLambda(node.getMulticutSep1D().getFixp())); // y value
-            		continue;
-            	}
-            	String tmp = l.getName();
-            	assert(count < 2);
-            	// two potential substitutions
-            	tmp = tmp.replaceAll("_MASK_1", "CA");
-            	tmp = tmp.replaceAll("_MASK_2", "CB");
-            	names[(count+1)%2] = tmp; // the upper layer must be first in names. First layer is alwayss the bottom
-                EdgeH leftEdge = node.getLeftEdge();
-                EdgeH rightEdge = node.getRightEdge();
-                EdgeV topEdge = node.getTopEdge();
-                EdgeV bottomEdge = node.getBottomEdge();
-                long portLowX = leftEdge.getFixpValue(size);
-                long portHighX = rightEdge.getFixpValue(size);
-                long portLowY = bottomEdge.getFixpValue(size);
-                long portHighY = topEdge.getFixpValue(size);
-                xExt[count] = FixpCoord.fixpToLambda(portHighX-portLowX);
-                yExt[count] = FixpCoord.fixpToLambda(portHighY-portLowY);
-            	count++;
-            	
-            }
-            // do extra process with vias once values have been identified
-            for (int i = 0; i < 2; i++)
-            {
-            	xExt[i] = (xExt[i] - viaSize[0])*scale/2;
-            	yExt[i] = (yExt[i] - viaSize[1])*scale/2;
-            }
-            
-            // using -1 as number of fractions to force zero digits
-            String newName = names[0] + "_" + names[1] + "_"; // first the upper in names
-            newName += "X_" 
-            		+ TextUtils.formatDouble(viaSize[0]*scale, -1) + "_" + TextUtils.formatDouble(viaSize[1]*scale, -1) 
-            		+ "_1_1_" 
-            		+ TextUtils.formatDouble(viaSpacing[0]*scale, -1) + "_" + TextUtils.formatDouble(viaSpacing[1]*scale, -1) + "_"
-            		+ TextUtils.formatDouble(xExt[0], -1) + "_" + TextUtils.formatDouble(yExt[0], -1) + "_" 
-            		+ TextUtils.formatDouble(xExt[1], -1) + "_" + TextUtils.formatDouble(yExt[1], -1);
-            //System.out.println(" name " + newName);
-            //System.out.println(s + "-" + TextUtils.formatDouble(wid, 0) + "-" + TextUtils.formatDouble(hei, 0));
-            //return s + "-" + TextUtils.formatDouble(wid, 0) + "-" + TextUtils.formatDouble(hei, 0);
-            return newName;
-        }
-        
-        CellName getDefaultCellName() {
-            return CellName.parseName(this + ";1{lay}");            
-        }
-    }
-    
-    private static class ContactTemplate
-    {
-        final CellId cellId;
-        final Orientation orient;
-        final ExportId exportId;
-        
-        ContactTemplate(CellId cellId, Orientation orient, ExportId exportId)
-        {
-            this.cellId = cellId;
-            this.orient = orient;
-            this.exportId = exportId;
-        }
-    }
-    
     private class MyShapeBuilder extends AbstractShapeBuilder {
         private final CellId cellId;
         private final TextDescriptor nameTd = ep.getNodeTextDescriptor();
         private final TextDescriptor protoTd = ep.getInstanceTextDescriptor();
         private CellBackup cellBackup;
         private final List<ImmutableNodeInst> nodes = new ArrayList<ImmutableNodeInst>();
-        
+
         MyShapeBuilder() {
             cellId = null;
             cellBackup = null;
         }
-        
+
         MyShapeBuilder(CellId cellId) {
             setup(oldTechPool);
             this.cellId = cellId;
@@ -733,11 +592,11 @@ class SeaOfGatesCellBuilder {
         int getNextNodeId() {
             return nodes.size();
         }
-        
+
         Name getNextName() {
             return Name.findName("plnode@" + getNextNodeId());
         }
-        
+
         @Override
         protected void addPoly(int numPoints, Poly.Type style, Layer layer, EGraphics graphicsOverride, PrimitivePort pp)
         {
@@ -789,7 +648,7 @@ class SeaOfGatesCellBuilder {
                 Orientation.IDENT, anchor, size,
                 0, 0, protoTd));
         }
-        
+
         CellBackup commit() {
             cellBackup = cellBackup.with(cellBackup.cellRevision.d,
                 nodes.toArray(new ImmutableNodeInst[nodes.size()]),
@@ -797,52 +656,6 @@ class SeaOfGatesCellBuilder {
             return cellBackup;
         }
     }
-    
-    private void makeGridBox(long[] gridCoords, EPoint tailLocation, boolean tailExtended, EPoint headLocation, boolean headExtended, long gridExtend)
-    {
-        long et = tailExtended ? gridExtend : 0;
-        long eh = headExtended ? gridExtend : 0;
-        long m;
-        long lx, ly, hx, hy;
-        int angle = GenMath.figureAngle(tailLocation, headLocation);
-        switch (angle) {
-            case -1:
-            case 0:
-                m = tailLocation.getGridY();
-                lx = tailLocation.getGridX() - et;
-                ly = m - gridExtend;
-                hx = headLocation.getGridX() + eh;
-                hy = m + gridExtend;
-                break;
-            case 900:
-                m = tailLocation.getGridX();
-                lx = m - gridExtend;
-                ly = tailLocation.getGridY() - et;
-                hx = m + gridExtend;
-                hy = headLocation.getGridY() + eh;
-                break;
-            case 1800:
-                m = tailLocation.getGridY();
-                lx = headLocation.getGridX() - eh;
-                ly = m - gridExtend;
-                hx = tailLocation.getGridX() + et;
-                hy = m + gridExtend;
-                break;
-            case 2700:
-                m = tailLocation.getGridX();
-                lx = m - gridExtend;
-                ly = headLocation.getGridY() - eh;
-                hx = m + gridExtend;
-                hy = tailLocation.getGridY() + et;
-                break;
-            default:
-                throw new AssertionError();
-        }
-        gridCoords[0] = lx;
-        gridCoords[1] = ly;
-        gridCoords[2] = hx;
-        gridCoords[3] = hy;
-    }
 
     Snapshot commit() {
         ImmutableNodeInst[] newNodes;
@@ -937,11 +750,11 @@ class SeaOfGatesCellBuilder {
         return index >= 0 ? index : -(index + 1);
     }
 
-    private int searchArcInsertionPoint(String basename) {
-        assert basename.endsWith("@0");
-        char nextChar = (char) (basename.charAt(basename.length() - 2) + 1);
-        String nextName = basename.substring(0, basename.length() - 2) + nextChar;
-        int index = oldCellRevision.arcs.searchByName(nextName);
-        return index >= 0 ? index : -(index + 1);
-    }
+//    private int searchArcInsertionPoint(String basename) {
+//        assert basename.endsWith("@0");
+//        char nextChar = (char) (basename.charAt(basename.length() - 2) + 1);
+//        String nextName = basename.substring(0, basename.length() - 2) + nextChar;
+//        int index = oldCellRevision.arcs.searchByName(nextName);
+//        return index >= 0 ? index : -(index + 1);
+//    }
 }
