@@ -4,7 +4,7 @@
  *
  * File: CircuitChanges.java
  *
- * Copyright (c) 2003, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, Static Free Software. All rights reserved.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -839,13 +839,24 @@ public class CircuitChanges
 
 		List<NodeInst> selected = MenuCommands.getSelectedNodes();
 		List<NodeInst> instances = new ArrayList<NodeInst>();
+		int schematicCells = 0;
 		for(NodeInst ni : selected)
 		{
-			if (ni.isCellInstance()) instances.add(ni);
+			if (!ni.isCellInstance()) continue;
+			if (!ni.getProto().getTechnology().isLayout())
+			{
+				schematicCells++;
+				continue;
+			}
+			instances.add(ni);
 		}
+		if (schematicCells > 0)
+			System.out.println("WARNING: Cannot extract non-layout cells (" + schematicCells + " selected)");
 		if (instances.size() == 0)
 		{
-			System.out.println("No cell instances are selected...no extraction done");
+			String msg = "No extraction done";
+			if (schematicCells == 0) msg = "No cell instances are selected..." + msg;
+			System.out.println(msg);
 			return;
 		}
 		new CellChangeJobs.ExtractCellInstances(cell, instances, depth, User.isExtractCopiesExports(),
