@@ -4,7 +4,7 @@
  *
  * File: Change.java
  *
- * Copyright (c) 2003, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, Static Free Software. All rights reserved.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -1179,7 +1179,7 @@ public class Change extends EModelessDialog implements HighlightListener
                 return;
             }
             PrimitiveNode.Function fun = func != null ? func : PrimitiveNode.Function.UNKNOWN;
-            BatchChanges.NodeReplacement nr = new BatchChanges.NodeReplacement(lNi, np, fun);
+            BatchChanges.NodeReplacement nr = new BatchChanges.NodeReplacement(lNi, np, fun, null);
             if (lNi.checkReplacement(nr, ep, ignorePortNames, allowMissingPorts)) {
                 replacements.put(lNi, nr);
             } else {
