@@ -4,7 +4,7 @@
  *
  * File: GDSTab.java
  *
- * Copyright (c) 2004, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2004, Static Free Software. All rights reserved.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,6 +34,7 @@ import com.sun.electric.util.TextUtils;
 
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
+import java.text.DecimalFormat;
 import java.util.Iterator;
 
 import javax.swing.DefaultListModel;
@@ -62,7 +63,8 @@ public class GDSTab extends PreferencePanel
     private Setting gdsOutWriteExportCharacteristicsSetting = IOTool.getGDSOutWriteExportCharacteristicsSetting();
     private Setting gdsCellNameLenMaxSetting = IOTool.getGDSCellNameLenMaxSetting();
     private Setting gdsInputScaleSetting = IOTool.getGDSInputScaleSetting();
-    private Setting gdsOutputScaleSetting = IOTool.getGDSOutputScaleSetting();
+    private Setting gdsOutputPrecisionSetting = IOTool.getGDSOutputPrecisionSetting();
+    private Setting gdsOutputUnitsPerMeterSetting = IOTool.getGDSOutputUnitsPerMeterSetting();
 
     /** Creates new form GDSTab */
 	public GDSTab(PreferencesFrame parent, boolean modal)
@@ -82,6 +84,8 @@ public class GDSTab extends PreferencePanel
 	    EDialog.makeTextFieldSelectAllOnTab(gdsCellNameLenMax);
 	    EDialog.makeTextFieldSelectAllOnTab(gdsDefaultTextLayer);
 	    EDialog.makeTextFieldSelectAllOnTab(gdsInputScale);
+	    EDialog.makeTextFieldSelectAllOnTab(gdsOutputPrecision);
+	    EDialog.makeTextFieldSelectAllOnTab(gdsOutputUnitsPerMeter);
 	}
 
 	/** return the JPanel to use for the user preferences. */
@@ -101,7 +105,6 @@ public class GDSTab extends PreferencePanel
 	{
 		// user preferences
 		gdsInputMergesBoxes.setSelected(IOTool.isGDSInMergesBoxes());
-		gdsIncludesText.setSelected(IOTool.isGDSIncludesText());
 		gdsInputExpandsCells.setSelected(IOTool.isGDSInExpandsCells());
         gdsSimplifyCells.setSelected(IOTool.isGDSInSimplifyCells());
 		gdsInputInstantiatesArrays.setSelected(IOTool.isGDSInInstantiatesArrays());
@@ -114,10 +117,12 @@ public class GDSTab extends PreferencePanel
         gdsUnknownLayers.addItem("Convert to random layer");
         gdsUnknownLayers.setSelectedIndex(IOTool.getGDSInUnknownLayerHandling());
         gdsCadenceCompatibility.setSelected(IOTool.isGDSCadenceCompatibility());
+        gdsDumpText.setSelected(IOTool.isGDSDumpReadable());
         gdsExportAllCells.setSelected(IOTool.isGDSWritesEntireLibrary());
         gdsExportFlatDesign.setSelected(IOTool.isGDSFlatDesign());
         gdsConvertNCCExportsConnectedByParentPins.setSelected(IOTool.getGDSConvertNCCExportsConnectedByParentPins());
         gdsVisibility.setSelected(IOTool.isGDSOnlyInvisibleLayers());
+		gdsIncludesText.setSelected(IOTool.isGDSIncludesText());
 
         // project preferences
 		gdsOutputMergesBoxes.setSelected(getBoolean(gdsOutMergesBoxesSetting));
@@ -129,7 +134,10 @@ public class GDSTab extends PreferencePanel
         gdsOutputConvertsBracketsInExports.setSelected(getBoolean(gdsOutputConvertsBracketsInExportsSetting));
         gdsCellNameLenMax.setText(Integer.toString(getInt(gdsCellNameLenMaxSetting)));
         gdsInputScale.setText(TextUtils.formatDouble(getDouble(gdsInputScaleSetting), 0));
-        gdsOutputScale.setText(TextUtils.formatDouble(getDouble(gdsOutputScaleSetting), 0));
+        gdsOutputPrecision.setText(TextUtils.formatDouble(IOTool.getGDSOutputPrecision()));
+        DecimalFormat myFormatter = new DecimalFormat("#,###");
+        String output = myFormatter.format(IOTool.getGDSOutputUnitsPerMeter());
+        gdsOutputUnitsPerMeter.setText(output);
 
         // build the layers list
 		gdsLayersModel = new DefaultListModel();
@@ -211,6 +219,9 @@ public class GDSTab extends PreferencePanel
 		currentValue = gdsCadenceCompatibility.isSelected();
 		if (currentValue != IOTool.isGDSCadenceCompatibility())
 			IOTool.setGDSCadenceCompatibility(currentValue);
+		currentValue = gdsDumpText.isSelected();
+		if (currentValue != IOTool.isGDSDumpReadable())
+			IOTool.setGDSDumpReadable(currentValue);
 		currentValue = gdsExportAllCells.isSelected();
 		if (currentValue != IOTool.isGDSWritesEntireLibrary())
 			IOTool.setGDSWritesEntireLibrary(currentValue);
@@ -234,7 +245,8 @@ public class GDSTab extends PreferencePanel
         setBoolean(gdsOutWriteExportCharacteristicsSetting, gdsOutputWritesCharacteristics.isSelected());        
         setInt(gdsCellNameLenMaxSetting, TextUtils.atoi(gdsCellNameLenMax.getText()));
         setDouble(gdsInputScaleSetting, TextUtils.atof(gdsInputScale.getText()));
-        setDouble(gdsOutputScaleSetting, TextUtils.atof(gdsOutputScale.getText()));
+        setDouble(gdsOutputPrecisionSetting, TextUtils.atof(gdsOutputPrecision.getText()));
+        setDouble(gdsOutputUnitsPerMeterSetting, TextUtils.atof(gdsOutputUnitsPerMeter.getText()));
     }
 
     /**
@@ -254,8 +266,6 @@ public class GDSTab extends PreferencePanel
 		// user preferences
 		if (IOTool.isFactoryGDSInMergesBoxes() != IOTool.isGDSInMergesBoxes())
 			IOTool.setGDSInMergesBoxes(IOTool.isFactoryGDSInMergesBoxes());
-		if (IOTool.isFactoryGDSIncludesText() != IOTool.isGDSIncludesText())
-			IOTool.setGDSIncludesText(IOTool.isFactoryGDSIncludesText());
 		if (IOTool.isFactoryGDSInExpandsCells() != IOTool.isGDSInExpandsCells())
 			IOTool.setGDSInExpandsCells(IOTool.isFactoryGDSInExpandsCells());
 		if (IOTool.isFactoryGDSInSimplifyCells() != IOTool.isGDSInSimplifyCells())
@@ -268,12 +278,18 @@ public class GDSTab extends PreferencePanel
 			IOTool.setGDSInUnknownLayerHandling(IOTool.getFactoryGDSInUnknownLayerHandling());
 		if (IOTool.isFactoryGDSCadenceCompatibility() != IOTool.isGDSCadenceCompatibility())
 			IOTool.setGDSCadenceCompatibility(IOTool.isFactoryGDSCadenceCompatibility());
+		if (IOTool.isFactoryGDSDumpReadable() != IOTool.isGDSDumpReadable())
+			IOTool.setGDSDumpReadable(IOTool.isFactoryGDSDumpReadable());
 		if (IOTool.isFactoryGDSWritesEntireLibrary() != IOTool.isGDSWritesEntireLibrary())
 			IOTool.setGDSWritesEntireLibrary(IOTool.isFactoryGDSWritesEntireLibrary());
 		if (IOTool.isFactoryGDSFlatDesign() != IOTool.isGDSFlatDesign())
 			IOTool.setGDSFlatDesign(IOTool.isFactoryGDSFlatDesign());
 		if (IOTool.getFactoryGDSConvertNCCExportsConnectedByParentPins() != IOTool.getGDSConvertNCCExportsConnectedByParentPins())
 			IOTool.setGDSConvertNCCExportsConnectedByParentPins(IOTool.getFactoryGDSConvertNCCExportsConnectedByParentPins());
+		if (IOTool.isFactoryGDSOnlyInvisibleLayers() != IOTool.isGDSOnlyInvisibleLayers())
+			IOTool.setGDSOnlyInvisibleLayers(IOTool.isFactoryGDSOnlyInvisibleLayers());
+		if (IOTool.isFactoryGDSIncludesText() != IOTool.isGDSIncludesText())
+			IOTool.setGDSIncludesText(IOTool.isFactoryGDSIncludesText());
 
         // project preferences
         Foundry foundry = (Foundry)foundrySelection.getSelectedItem();
@@ -297,7 +313,8 @@ public class GDSTab extends PreferencePanel
         setBoolean(gdsOutWriteExportCharacteristicsSetting, ((Boolean)gdsOutWriteExportCharacteristicsSetting.getFactoryValue()).booleanValue());
         setInt(gdsCellNameLenMaxSetting, ((Integer)gdsCellNameLenMaxSetting.getFactoryValue()).intValue());
         setInt(gdsOutDefaultTextLayerSetting, ((Integer)gdsOutDefaultTextLayerSetting.getFactoryValue()).intValue());
-        setDouble(gdsOutputScaleSetting, ((Double)gdsOutputScaleSetting.getFactoryValue()).doubleValue());
+        setDouble(gdsOutputPrecisionSetting, ((Double)gdsOutputPrecisionSetting.getFactoryValue()).doubleValue());
+        setDouble(gdsOutputUnitsPerMeterSetting, ((Double)gdsOutputUnitsPerMeterSetting.getFactoryValue()).doubleValue());
 	}
 
     private void foundryChanged()
@@ -563,6 +580,7 @@ public class GDSTab extends PreferencePanel
         jLabel1 = new javax.swing.JLabel();
         gdsUnknownLayers = new javax.swing.JComboBox();
         gdsCadenceCompatibility = new javax.swing.JCheckBox();
+        gdsDumpText = new javax.swing.JCheckBox();
         Export = new javax.swing.JPanel();
         gdsExportAllCells = new javax.swing.JCheckBox();
         gdsExportFlatDesign = new javax.swing.JCheckBox();
@@ -582,10 +600,12 @@ public class GDSTab extends PreferencePanel
         gdsOutputConvertsBracketsInExports = new javax.swing.JCheckBox();
         jLabel5 = new javax.swing.JLabel();
         gdsCellNameLenMax = new javax.swing.JTextField();
-        jLabel12 = new javax.swing.JLabel();
-        gdsOutputScale = new javax.swing.JTextField();
         gdsOutputColapseNames = new javax.swing.JCheckBox();
         gdsOutputWritesCharacteristics = new javax.swing.JCheckBox();
+        jLabel15 = new javax.swing.JLabel();
+        gdsOutputPrecision = new javax.swing.JTextField();
+        jLabel16 = new javax.swing.JLabel();
+        gdsOutputUnitsPerMeter = new javax.swing.JTextField();
         inputPanel = new javax.swing.JPanel();
         jLabel11 = new javax.swing.JLabel();
         gdsInputScale = new javax.swing.JTextField();
@@ -700,6 +720,15 @@ public class GDSTab extends PreferencePanel
         gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 4);
         Import.add(gdsCadenceCompatibility, gridBagConstraints);
 
+        gdsDumpText.setText("Dump readable data while reading");
+        gridBagConstraints = new java.awt.GridBagConstraints();
+        gridBagConstraints.gridx = 0;
+        gridBagConstraints.gridy = 7;
+        gridBagConstraints.gridwidth = 2;
+        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
+        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 4);
+        Import.add(gdsDumpText, gridBagConstraints);
+
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 0;
         gridBagConstraints.gridy = 1;
@@ -714,6 +743,7 @@ public class GDSTab extends PreferencePanel
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 0;
         gridBagConstraints.gridy = 0;
+        gridBagConstraints.gridwidth = 2;
         gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
         gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 4);
         Export.add(gdsExportAllCells, gridBagConstraints);
@@ -722,6 +752,7 @@ public class GDSTab extends PreferencePanel
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 0;
         gridBagConstraints.gridy = 1;
+        gridBagConstraints.gridwidth = 2;
         gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
         gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 4);
         Export.add(gdsExportFlatDesign, gridBagConstraints);
@@ -730,6 +761,7 @@ public class GDSTab extends PreferencePanel
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 0;
         gridBagConstraints.gridy = 2;
+        gridBagConstraints.gridwidth = 2;
         gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
         gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 4);
         Export.add(gdsConvertNCCExportsConnectedByParentPins, gridBagConstraints);
@@ -849,7 +881,7 @@ public class GDSTab extends PreferencePanel
         gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 4);
         outputPanel.add(gdsOutputConvertsBracketsInExports, gridBagConstraints);
 
-        jLabel5.setText("Max chars in output cell name:");
+        jLabel5.setText("Max chars in cell name:");
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 0;
         gridBagConstraints.gridy = 6;
@@ -864,23 +896,6 @@ public class GDSTab extends PreferencePanel
         gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 4);
         outputPanel.add(gdsCellNameLenMax, gridBagConstraints);
 
-        jLabel12.setText("Scale by:");
-        gridBagConstraints = new java.awt.GridBagConstraints();
-        gridBagConstraints.gridx = 0;
-        gridBagConstraints.gridy = 7;
-        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
-        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 4);
-        outputPanel.add(jLabel12, gridBagConstraints);
-
-        gdsOutputScale.setColumns(8);
-        gridBagConstraints = new java.awt.GridBagConstraints();
-        gridBagConstraints.gridx = 1;
-        gridBagConstraints.gridy = 7;
-        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
-        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
-        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 4);
-        outputPanel.add(gdsOutputScale, gridBagConstraints);
-
         gdsOutputColapseNames.setText("Output collapses VDD/GND pin names");
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 0;
@@ -899,6 +914,40 @@ public class GDSTab extends PreferencePanel
         gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 4);
         outputPanel.add(gdsOutputWritesCharacteristics, gridBagConstraints);
 
+        jLabel15.setText("Precision:");
+        gridBagConstraints = new java.awt.GridBagConstraints();
+        gridBagConstraints.gridx = 0;
+        gridBagConstraints.gridy = 7;
+        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
+        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 4);
+        outputPanel.add(jLabel15, gridBagConstraints);
+
+        gdsOutputPrecision.setColumns(8);
+        gridBagConstraints = new java.awt.GridBagConstraints();
+        gridBagConstraints.gridx = 1;
+        gridBagConstraints.gridy = 7;
+        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
+        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
+        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 4);
+        outputPanel.add(gdsOutputPrecision, gridBagConstraints);
+
+        jLabel16.setText("Units/meter:");
+        gridBagConstraints = new java.awt.GridBagConstraints();
+        gridBagConstraints.gridx = 0;
+        gridBagConstraints.gridy = 8;
+        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
+        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 4);
+        outputPanel.add(jLabel16, gridBagConstraints);
+
+        gdsOutputUnitsPerMeter.setColumns(8);
+        gridBagConstraints = new java.awt.GridBagConstraints();
+        gridBagConstraints.gridx = 1;
+        gridBagConstraints.gridy = 8;
+        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
+        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
+        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 4);
+        outputPanel.add(gdsOutputUnitsPerMeter, gridBagConstraints);
+
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 2;
         gridBagConstraints.gridy = 4;
@@ -1113,6 +1162,7 @@ public class GDSTab extends PreferencePanel
     private javax.swing.JTextField gdsCellNameLenMax;
     private javax.swing.JCheckBox gdsConvertNCCExportsConnectedByParentPins;
     private javax.swing.JTextField gdsDefaultTextLayer;
+    private javax.swing.JCheckBox gdsDumpText;
     private javax.swing.JCheckBox gdsExportAllCells;
     private javax.swing.JCheckBox gdsExportFlatDesign;
     private javax.swing.JLabel gdsFoundryName;
@@ -1129,7 +1179,8 @@ public class GDSTab extends PreferencePanel
     private javax.swing.JCheckBox gdsOutputColapseNames;
     private javax.swing.JCheckBox gdsOutputConvertsBracketsInExports;
     private javax.swing.JCheckBox gdsOutputMergesBoxes;
-    private javax.swing.JTextField gdsOutputScale;
+    private javax.swing.JTextField gdsOutputPrecision;
+    private javax.swing.JTextField gdsOutputUnitsPerMeter;
     private javax.swing.JCheckBox gdsOutputUpperCase;
     private javax.swing.JCheckBox gdsOutputWritesCharacteristics;
     private javax.swing.JCheckBox gdsOutputWritesExportPins;
@@ -1144,9 +1195,10 @@ public class GDSTab extends PreferencePanel
     private javax.swing.JLabel jLabel1;
     private javax.swing.JLabel jLabel10;
     private javax.swing.JLabel jLabel11;
-    private javax.swing.JLabel jLabel12;
     private javax.swing.JLabel jLabel13;
     private javax.swing.JLabel jLabel14;
+    private javax.swing.JLabel jLabel15;
+    private javax.swing.JLabel jLabel16;
     private javax.swing.JLabel jLabel2;
     private javax.swing.JLabel jLabel3;
     private javax.swing.JLabel jLabel4;
