@@ -4,7 +4,7 @@
  *
  * File: CellMenu.java
  *
- * Copyright (c) 2003, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, Static Free Software. All rights reserved.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,9 +21,9 @@
  */
 package com.sun.electric.tool.user.menus;
 
-import com.sun.electric.database.EditingPreferences;
 import static com.sun.electric.tool.user.menus.EMenuItem.SEPARATOR;
 
+import com.sun.electric.database.EditingPreferences;
 import com.sun.electric.database.geometry.EPoint;
 import com.sun.electric.database.geometry.ERectangle;
 import com.sun.electric.database.hierarchy.Cell;
@@ -73,6 +73,8 @@ import java.awt.geom.Rectangle2D;
 import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Map;
+import java.util.TreeMap;
 
 import javax.swing.JButton;
 import javax.swing.JLabel;
@@ -508,7 +510,7 @@ public class CellMenu {
 		GeometrySearch search = new GeometrySearch(curEdit.getLayerVisibility(), curEdit.getGraphicsPreferences());
 		List<GeometrySearch.GeometrySearchResult> possibleTargets = search.searchGeometries(cell, point, true);
 
-		// eliminate results at the top level and duplicate results at any lower level
+		// eliminate results at the top level
 		for(int i=0; i<possibleTargets.size(); i++)
 		{
 			GeometrySearch.GeometrySearchResult res = possibleTargets.get(i);
@@ -521,17 +523,21 @@ public class CellMenu {
 			}
 
 			// also remove duplicate contexts at lower levels
-			for(int j=0; j<i; j++)
+final boolean JUSTONEATALEVEL = false;
+			if (JUSTONEATALEVEL)
 			{
-				GeometrySearch.GeometrySearchResult oRes = possibleTargets.get(j);
-				if (sameContext(oRes.getContext(), res.getContext()))
+				for(int j=0; j<i; j++)
 				{
-					possibleTargets.remove(i);
-					i--;
+					GeometrySearch.GeometrySearchResult oRes = possibleTargets.get(j);
+					if (sameContext(oRes.getContext(), res.getContext()))
+					{
+						possibleTargets.remove(i);
+						i--;
+					}
 				}
 			}
 		}
-
+		
 		// give error if nothing was found
 		if (possibleTargets.size() == 0)
 		{
@@ -548,16 +554,32 @@ public class CellMenu {
 			descendToObject(possibleTargets.get(0));
 		} else
 		{
-			// let the user choose
-            JPopupMenu menu = new JPopupMenu();
+			// let the user choose, sort by the lower level
+			Map<String,List<GeometrySearch.GeometrySearchResult>> levelMap = new TreeMap<String,List<GeometrySearch.GeometrySearchResult>>();
+            for(GeometrySearch.GeometrySearchResult res : possibleTargets)
+			{
+            	String resDescript = res.getInstPath(res.getContext());
+            	List<GeometrySearch.GeometrySearchResult> itemsAtLevel = levelMap.get(resDescript);
+            	if (itemsAtLevel == null) levelMap.put(resDescript, itemsAtLevel = new ArrayList<GeometrySearch.GeometrySearchResult>());
+            	itemsAtLevel.add(res);
+			}
+
+			JPopupMenu menu = new JPopupMenu();
         	JMenuItem menuItem = new JMenuItem("Multiple objects under the cursor...choose one");
             menu.add(menuItem);
             menu.addSeparator();
-            for(GeometrySearch.GeometrySearchResult res : possibleTargets)
+            for(String resDescript : levelMap.keySet())
             {
-            	menuItem = new JMenuItem(res.describe());
-            	menuItem.addActionListener(new DownHierToObjectActionListener(res));
+            	List<GeometrySearch.GeometrySearchResult> itemsAtLevel = levelMap.get(resDescript);
+            	menuItem = new JMenuItem("ITEMS IN: " + resDescript);
+            	menuItem.setEnabled(false);
                 menu.add(menuItem);
+                for(GeometrySearch.GeometrySearchResult res : itemsAtLevel)
+                {
+                	menuItem = new JMenuItem("     " + res.getGeometric().describe(false));
+                	menuItem.addActionListener(new DownHierToObjectActionListener(res));
+                    menu.add(menuItem);
+                }
             }
             menu.show(curEdit, 100, 100);
 		}
