@@ -4,7 +4,7 @@
  *
  * File: EditMenu.java
  *
- * Copyright (c) 2003, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, Static Free Software. All rights reserved.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@ import com.sun.electric.database.EditingPreferences;
 import com.sun.electric.database.change.DatabaseChangeEvent;
 import com.sun.electric.database.change.Undo;
 import com.sun.electric.database.geometry.EPoint;
+import com.sun.electric.database.geometry.ScreenPoint;
 import com.sun.electric.database.hierarchy.Cell;
 import com.sun.electric.database.hierarchy.Export;
 import com.sun.electric.database.hierarchy.Library;
@@ -51,6 +52,7 @@ import com.sun.electric.tool.Job;
 import com.sun.electric.tool.JobException;
 import com.sun.electric.tool.io.FileType;
 import com.sun.electric.tool.io.IOTool;
+import com.sun.electric.tool.io.output.GDS;
 import com.sun.electric.tool.sandbox.TechExplorerDriver;
 import com.sun.electric.tool.user.ActivityLogger;
 import com.sun.electric.tool.user.CircuitChangeJobs;
@@ -99,8 +101,12 @@ import com.sun.electric.tool.user.ui.WindowFrame;
 import com.sun.electric.tool.user.waveform.WaveformWindow;
 import com.sun.electric.util.math.DBMath;
 
+import java.awt.Component;
 import java.awt.Dimension;
+import java.awt.Point;
 import java.awt.Toolkit;
+import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
 import java.awt.event.InputEvent;
 import java.awt.event.KeyEvent;
 import java.awt.event.MouseEvent;
@@ -114,7 +120,9 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Set;
 
+import javax.swing.JMenuItem;
 import javax.swing.JOptionPane;
+import javax.swing.JPopupMenu;
 import javax.swing.KeyStroke;
 
 /**
@@ -310,7 +318,7 @@ public class EditMenu {
 
 			ToolBar.modesSubMenu, // O
 
-		// mnemonic keys available: AB    GH JKLMNOPQRS UVWXYZ
+		// mnemonic keys available: AB      JKLMNOPQRS UVWXYZ
 			new EMenu("Te_xt",
 				new EMenuItem("_Find Text...", 'L') { public void run() {
 					FindText.findTextDialog(); }},
@@ -322,6 +330,10 @@ public class EditMenu {
 					changeGlobalTextSize(0.8); }},
 				new EMenuItem("Add _Text Annotation", KeyStroke.getKeyStroke('T', 0)) { public void run() {
 					PaletteFrame.placeInstance("ART_message", null, false); }},
+				new EMenuItem("Add _GDS Text") { public void run() {
+					addGDSText(false); }},
+				new EMenuItem("Add _HV GDS Text") { public void run() {
+					addGDSText(true); }},
 				new EMenuItem("Edit Text Cell _Externally...") { public void run() {
 					ExternalEditing.editExternally(); }}),
 
@@ -430,7 +442,7 @@ public class EditMenu {
 				new EMenuItem("Technology Creation _Wizard...") { public void run() {
 					TechEditWizard.techEditWizardCommand(); }}),
 
-		// mnemonic keys available:      F    K     Q        Z
+		// mnemonic keys available:                 Q        Z
 			new EMenu("_Selection",
 				new EMenuItem("Sele_ct All", 'A') { public void run() {
 					selectAllCommand(); }},
@@ -450,10 +462,12 @@ public class EditMenu {
 				SEPARATOR,
 				new EMenuItem("Select O_bject...") { public void run() {
 					SelectObject.selectObjectDialog(null, false); }},
+				new EMenuItem("Select Object Unde_r Cursor...", KeyStroke.getKeyStroke('U', 0)) { public void run() {
+					selectObjectUnderCursor(); }},
 				new EMenuItem("Deselect All _Arcs") { public void run() {
 					deselectAllArcsCommand(); }},
 				SEPARATOR,
-				new EMenuItem("Make Selected Eas_y") { public void run() {
+				new EMenuItem("Ma_ke Selected Easy") { public void run() {
 					selectMakeEasyCommand(); }},
 				new EMenuItem("Make Selected Har_d") { public void run() {
 					selectMakeHardCommand(); }},
@@ -476,7 +490,7 @@ public class EditMenu {
 					showNextErrorCommand(false); }},
 				new EMenuItem("Show Previous Error, sa_me Window", KeyStroke.getKeyStroke('[')) { public void run() {
 					showPrevErrorCommand(false); }},
-                new EMenuItem("Show Single Geometry", KeyStroke.getKeyStroke(KeyEvent.VK_SLASH, 0)) { public void run() {
+                new EMenuItem("Show Single Geometr_y", KeyStroke.getKeyStroke(KeyEvent.VK_SLASH, 0)) { public void run() {
 					showSingleGeometryCommand(); }},
                 new EMenuItem("Show Current Co_llection of Errors") { public void run() {
 					ErrorLoggerTree.showCurrentErrors(); }},
@@ -485,7 +499,7 @@ public class EditMenu {
 					addToWaveformNewCommand(); }},
 				new EMenuItem("Add to _Waveform in Current Panel", KeyStroke.getKeyStroke('O', 0)) { public void run() {
 					addToWaveformCurrentCommand(); }},
-				new EMenuItem("_Remove from Waveform", KeyStroke.getKeyStroke('R', 0)) { public void run() {
+				new EMenuItem("Remove _from Waveform", KeyStroke.getKeyStroke('R', 0)) { public void run() {
 					removeFromWaveformCommand(); }}));
 	}
 
@@ -794,6 +808,50 @@ public class EditMenu {
 		}
 	}
 
+	/**
+	 * Method to add GDS text to the selected object.
+	 */
+	public static void addGDSText(boolean highV)
+	{
+		EditWindow wnd = EditWindow.needCurrent();
+		if (wnd == null) return;
+		Highlighter high = wnd.getHighlighter();
+		Highlight h = high.getOneHighlight();
+		if (h == null) return;
+		ElectricObject obj = h.getElectricObject();
+		if (obj == null) return;
+		if (obj instanceof PortInst)
+			obj = ((PortInst)obj).getNodeInst();
+		new AddGDSText(obj, highV);
+	}
+
+	private static class AddGDSText extends Job
+	{
+		private ElectricObject obj;
+		private boolean highV;
+
+		AddGDSText(ElectricObject obj, boolean highV)
+		{
+			super("Add GDS Text", User.getUserTool(), Job.Type.CHANGE, null, null, Job.Priority.USER);
+			this.obj = obj;
+			this.highV = highV;
+			startJob();
+		}
+
+        @Override
+		public boolean doIt() throws JobException
+        {
+        	if (highV)
+        	{
+        		obj.newDisplayVar(GDS.GDS_TEXT_HV_KEY, "GDS HV Text", getEditingPreferences());
+        	} else
+        	{
+        		obj.newDisplayVar(GDS.GDS_TEXT_KEY, "GDS Text", getEditingPreferences());
+        	}
+			return true;
+		}
+	}
+
 	/**
 	 * This method implements the command to highlight all objects in the current Cell.
 	 */
@@ -1215,6 +1273,49 @@ public class EditMenu {
 		System.out.println("Cannot advance the current selection");
 	}
 
+	/**
+	 * Command to popup a list of objects under the cursor and let the user choose one to select.
+	 */
+	private static JPopupMenu selectUnderCursorMenu = null;
+	private static Highlighter selectUnderCursorHighlighter = null;
+
+	public static void selectObjectUnderCursor()
+	{
+		EditWindow wnd = EditWindow.needCurrent();
+		if (wnd == null) return;
+		Point pt = wnd.getLastMousePosition();
+		Point2D center = wnd.screenToDatabase(pt.x, pt.y);
+		selectUnderCursorHighlighter = wnd.getHighlighter();
+		Rectangle2D bounds =  new Rectangle2D.Double(center.getX(), center.getY(), 0, 0);
+		boolean findSpecial = ToolBar.isSelectSpecial();
+		List<Highlight> underCursor = Highlighter.findAllInArea(selectUnderCursorHighlighter, wnd.getCell(), false, false, false, findSpecial, false, bounds, wnd);
+		boolean empty = underCursor.isEmpty();
+
+		selectUnderCursorMenu = new JPopupMenu("Select Object");
+
+		JMenuItem menuItemGetInfo = new JMenuItem(empty ? "Nothing Under Cursor To Select" : "Object to Select:");
+		selectUnderCursorMenu.add(menuItemGetInfo);
+		if (!empty) selectUnderCursorMenu.addSeparator();
+
+		for(Highlight h : underCursor)
+		{
+			final Geometric geom = h.getGeometric();
+			menuItemGetInfo = new JMenuItem(geom.describe(false));
+			menuItemGetInfo.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { showItem(geom); } });
+			selectUnderCursorMenu.add(menuItemGetInfo);
+		}
+
+		selectUnderCursorMenu.show(wnd.getPanel().getComponentAt((int)pt.x, (int)pt.y), (int)pt.x, (int)pt.y);
+	}
+
+	private static void showItem(Geometric geom)
+	{
+		selectUnderCursorHighlighter.clear();
+		selectUnderCursorHighlighter.addElectricObject(geom, geom.getParent());
+		selectUnderCursorHighlighter.finished();
+		selectUnderCursorMenu = null;
+	}
+
 	/**
      * This method implements the command to highlight nothing in the current Cell.
      */
