@@ -4,7 +4,7 @@
  *
  * File: FileMenu.java
  *
- * Copyright (c) 2003, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, Static Free Software. All rights reserved.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -243,6 +243,8 @@ public class FileMenu {
                     exportCommand(FileType.TELESIS, false); }},
                 new EMenuItem("DFTM...") { public void run() {
                     exportCommand(FileType.DFTM, false); }},
+                new EMenuItem("Flattened Rectangles...") { public void run() {
+                    exportCommand(FileType.FLATRECT, false); }},
                 SEPARATOR,
                 new EMenuItem("_Text Cell Contents...") { public void run() {
                     TextWindow.writeTextCell(); }},
@@ -443,6 +445,7 @@ public class FileMenu {
     public static void openLibraryCommand(URL file)
     {
         String fileName = file.getFile();
+        if (fileName.endsWith("/")) fileName = fileName.substring(0, fileName.length()-1);
         FileType defType = getLibraryFormat(fileName, null);
         if (defType == null) {
             // no valid extension, search for file with extension
@@ -479,7 +482,7 @@ public class FileMenu {
         if (defType == null) defType = FileType.DEFAULTLIB;
         File f = new File(file.getPath());
         if (defType != FileType.DELIB && f.isDirectory())
-            System.out.println("The filename provided is not a valid Electric's library: '" +
+            System.out.println("The filename provided is not a valid Electric library: '" +
             fileName + "'.");
         else
             new ReadLibrary(file, defType, TextUtils.getFilePath(file), null, null, null);
@@ -581,7 +584,7 @@ public class FileMenu {
 			}
             // read project preferences
             if (projsettings != null)
-                ProjSettings.readSettings(projsettings, getDatabase(), false);
+                ProjSettings.readSettings(projsettings, getDatabase(), false, false);
             HashSet<Library> oldLibs = new HashSet<Library>();
             for (Iterator<Library> it = getDatabase().getLibraries(); it.hasNext(); )
                 oldLibs.add(it.next());
@@ -1191,7 +1194,7 @@ public class FileMenu {
     }
 
     public static class RenameAndSaveLibraryTask implements Serializable {
-        private LibId libId;
+    	private LibId libId;
         private String newName;
         private FileType type;
         private boolean compatibleWith6;
@@ -1215,6 +1218,8 @@ public class FileMenu {
         	return new RenameAndSaveLibraryTask(lib, newName, type, compatibleWith6, backupScheme);
         }
         
+        public LibId getLidId() { return libId; }
+        		
         public IdMapper renameAndSave() throws JobException {
             IdMapper idMapper = null;
             boolean success = false;
@@ -1541,7 +1546,7 @@ public class FileMenu {
 				Rectangle2D cellBounds = ep.getRenderArea();
 				if (cellBounds == null) cellBounds = cell.getBounds();
 				LayerVisibility lv = new LayerVisibility(true);
-				BufferedImage img = EditWindow.getPrintImageFromData(ep, null, cellBounds, cell, null, lv);
+				BufferedImage img = EditWindow.getPrintImageFromData(ep, null, cellBounds, cell, null, lv, true);
 				PNG.writeImage(img, filePath);
 			} else
 			{
