@@ -4,7 +4,7 @@
  *
  * File: RoutingDebug.java
  *
- * Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, Static Free Software. All rights reserved.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,14 +21,17 @@
  */
 package com.sun.electric.tool.user.ui;
 
+import com.sun.electric.database.EditingPreferences;
 import com.sun.electric.database.geometry.EPoint;
 import com.sun.electric.database.geometry.ERectangle;
+import com.sun.electric.database.geometry.Poly;
 import com.sun.electric.database.geometry.PolyBase;
 import com.sun.electric.database.geometry.PolyBase.Point;
 import com.sun.electric.database.hierarchy.Cell;
 import com.sun.electric.database.text.TextUtils;
 import com.sun.electric.database.topology.ArcInst;
 import com.sun.electric.database.topology.PortInst;
+import com.sun.electric.database.topology.SteinerTree.SteinerTreePortPair;
 import com.sun.electric.technology.ArcProto;
 import com.sun.electric.technology.Layer;
 import com.sun.electric.technology.Technology;
@@ -44,6 +47,9 @@ import com.sun.electric.tool.routing.seaOfGates.SeaOfGatesEngine.GRNet;
 import com.sun.electric.tool.routing.seaOfGates.SeaOfGatesEngine.GRWire;
 import com.sun.electric.tool.routing.seaOfGates.SeaOfGatesEngine.GlobalRouter;
 import com.sun.electric.tool.routing.seaOfGates.SeaOfGatesEngine.NeededRoute;
+import com.sun.electric.tool.routing.seaOfGates.SeaOfGatesEngine.PossibleEndpoint;
+import com.sun.electric.tool.routing.seaOfGates.SeaOfGatesEngine.PossibleEndpoints;
+import com.sun.electric.tool.routing.seaOfGates.SeaOfGatesEngine.RoutesOnNetwork;
 import com.sun.electric.tool.routing.seaOfGates.SeaOfGatesEngine.SOGBound;
 import com.sun.electric.tool.routing.seaOfGates.SeaOfGatesEngine.SOGPoly;
 import com.sun.electric.tool.routing.seaOfGates.SeaOfGatesEngine.SearchVertex;
@@ -51,7 +57,6 @@ import com.sun.electric.tool.routing.seaOfGates.SeaOfGatesEngine.Wavefront;
 import com.sun.electric.tool.routing.seaOfGates.SeaOfGatesEngineFactory;
 import com.sun.electric.tool.routing.seaOfGates.SeaOfGatesEngineFactory.SeaOfGatesEngineType;
 import com.sun.electric.tool.routing.seaOfGates.SeaOfGatesHandlers;
-import com.sun.electric.tool.routing.seaOfGates.SeaOfGatesHandlers.Save;
 import com.sun.electric.tool.user.Highlight;
 import com.sun.electric.tool.user.Highlighter;
 import com.sun.electric.tool.user.User;
@@ -62,12 +67,14 @@ import com.sun.electric.tool.user.ui.ToolBar.CursorMode;
 import com.sun.electric.util.math.MutableInteger;
 
 import java.awt.Color;
+import java.awt.Component;
 import java.awt.Font;
 import java.awt.GridBagConstraints;
 import java.awt.GridBagLayout;
 import java.awt.Insets;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
+import java.awt.event.KeyEvent;
 import java.awt.event.MouseEvent;
 import java.awt.event.WindowAdapter;
 import java.awt.event.WindowEvent;
@@ -81,14 +88,17 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+import javax.swing.AbstractAction;
 import javax.swing.BorderFactory;
 import javax.swing.JButton;
 import javax.swing.JCheckBox;
 import javax.swing.JComboBox;
+import javax.swing.JComponent;
 import javax.swing.JLabel;
 import javax.swing.JPanel;
 import javax.swing.JTextArea;
 import javax.swing.JTextField;
+import javax.swing.KeyStroke;
 import javax.swing.UIManager;
 
 /**
@@ -98,6 +108,7 @@ public class RoutingDebug
 {
 	private static final double goalWidth = 0.005;
 	private static final double layerOffset = 0.04;
+	private static final double possibleGoalOffset = 3;
 
 	private static RoutingDialog debugDialog = null;
 	private static Set<SearchVertex> onPath = new HashSet<SearchVertex>();
@@ -110,7 +121,7 @@ public class RoutingDebug
 	private static Map<Integer,String> netNames;
 	private static int colorAssigned;
 	private static DebugType debuggingType;
-	private static enum DebugType {NONE, DISPLAYROUTING, REWIRENETS, RUNGLOBALROUTING};
+	private static enum DebugType {NONE, DISPLAYROUTING, REWIRENETS, SHOWSPINES, RUNGLOBALROUTING};
 
 	private static Color[] allColors = new Color[] {
 		new Color(65535>>8, 16385>>8, 16385>>8),	// Red
@@ -230,8 +241,8 @@ public class RoutingDebug
 			Point2D p2 = new Point2D.Double(nr.getBX(), nr.getBY());
 			highlighter.addLine(p1, p2, cell, true, Color.GREEN, false);
 
-			// show goal
-			debugDialog.showPathToGoal(result, cell, highlighter);
+			// show goal(s)
+			debugDialog.showPathToGoal(nr, result, cell, highlighter);
 
 			// remember path to goal
 			onPath.clear();
@@ -584,6 +595,138 @@ public class RoutingDebug
 		wnd.repaint();
 	}
 
+	/************************************* REWIRE NETWORKS *************************************/
+
+	private static List<RoutesOnNetwork> allSpineRoutes = null;
+
+	private static void showSpines()
+	{
+		EditWindow wnd = EditWindow.getCurrent();
+		Cell cell = wnd.getCell();
+		if (cell == null) return;
+		highlighter = wnd.getRulerHighlighter();
+		allSpineRoutes = null;
+		String routeName = debugDialog.whichOne.getText().trim();
+		if (routeName.length() == 0)
+		{
+			Job.getUserInterface().showErrorMessage("Must set name of route first in 'Route to Debug' field.", "Missing Information");
+			return;
+		}
+		new ShowSpineNets(cell, routeName);
+	}
+
+	public static class ShowSpineNets extends Job
+	{
+		private Cell cell;
+		private EditingPreferences ep;
+		private String routeName;
+
+		public ShowSpineNets(Cell c, String rn)
+		{
+			super("Show Spines", User.getUserTool(), Job.Type.CHANGE, null, null, Job.Priority.USER);
+			cell = c;
+			routeName = rn;
+			ep = UserInterfaceMain.getEditingPreferences();
+			startJob();
+		}
+
+		public boolean doIt() throws JobException
+		{
+			debuggingType = DebugType.SHOWSPINES;
+			debugDialog.router = SeaOfGatesEngineFactory.createSeaOfGatesEngine(SeaOfGatesEngineType.defaultVersion);
+			SeaOfGatesOptions prefs = new SeaOfGatesOptions();
+			prefs.getOptionsFromPreferences(false);
+			debugDialog.router.setPrefs(prefs);
+			debugDialog.svInfo = new HashMap<SearchVertex,SVState>();
+			netNames = new HashMap<Integer,String>();
+			svOrder = 1;
+
+			Job job = Job.getRunningJob();
+			debugDialog.router.routeIt(SeaOfGatesHandlers.getDefault(cell, debugDialog.router.getPrefs().resultCellName,
+				debugDialog.router.getPrefs().contactPlacementAction, job, ep), cell, false, null);
+			return true;
+		}
+
+		private static final int TAPSIZE = 8;
+		private static final int TAPOFFSET = 25;
+
+		@Override
+		public void terminateOK()
+		{
+			EditWindow wnd = EditWindow.getCurrent();
+			Highlighter h = wnd.getRulerHighlighter();
+			h.clear();
+			for(RoutesOnNetwork ron : allSpineRoutes)
+			{
+				if (!ron.getName().equals(routeName)) continue;
+
+				List<SteinerTreePortPair> pairs = ron.getPairs();
+				for(SteinerTreePortPair pair : pairs)
+				{
+					List<PortInst> taps = pair.getSpineTaps();
+					double fX = pair.getPort1().getCenter().getX(), fY = pair.getPort1().getCenter().getY();
+					double tX = pair.getPort2().getCenter().getX(), tY = pair.getPort2().getCenter().getY();
+					boolean hor = Math.abs(fX-tX) > Math.abs(fY-tY);
+					double offX = 0, offY = 0;
+					if (hor)
+					{
+						offY = TAPOFFSET;
+						fY = tY = Math.max(fY, tY);
+					} else
+					{
+						offX = -TAPOFFSET;
+						fX = tX = Math.min(fX, tX);
+					}
+
+					Color col = taps == null ? Color.GREEN : Color.RED;
+
+					h.addLine(EPoint.fromLambda(fX+offX, fY+offY), EPoint.fromLambda(tX+offX, tY+offY), cell, true, col, false);
+
+					// show the end points
+					Poly poly = new Poly(new Rectangle2D.Double(pair.getPort1().getCenter().getX()-TAPSIZE, pair.getPort1().getCenter().getY()-TAPSIZE, TAPSIZE*2, TAPSIZE*2));
+					poly.setStyle(Poly.Type.FILLED);
+					h.addPoly(poly, cell, Color.RED);
+					h.addLine(EPoint.fromLambda(fX+offX, fY+offY), pair.getPort1().getCenter(), cell, true, col, false);
+
+					poly = new Poly(new Rectangle2D.Double(pair.getPort2().getCenter().getX()-TAPSIZE, pair.getPort2().getCenter().getY()-TAPSIZE, TAPSIZE*2, TAPSIZE*2));
+					poly.setStyle(Poly.Type.FILLED);
+					h.addPoly(poly, cell, Color.RED);
+					h.addLine(EPoint.fromLambda(tX+offX, tY+offY), pair.getPort2().getCenter(), cell, true, col, false);
+
+					if (taps == null) continue;
+					for(PortInst pi : taps)
+					{
+						EPoint pt = pi.getCenter();
+						poly = new Poly(new Rectangle2D.Double(pt.getX()-TAPSIZE, pt.getY()-TAPSIZE, TAPSIZE*2, TAPSIZE*2));
+						poly.setStyle(Poly.Type.FILLED);
+						h.addPoly(poly, cell, Color.RED);
+						if (hor)
+						{
+							h.addLine(EPoint.fromLambda(pt.getX(), pt.getY()), EPoint.fromLambda(pt.getX(), tY+offY), cell, true, col, false);
+						} else
+						{
+							h.addLine(EPoint.fromLambda(pt.getX(), pt.getY()), EPoint.fromLambda(tX+offX, pt.getY()), cell, true, col, false);
+						}
+					}
+				}
+			}
+			h.finished();
+			wnd.repaint();
+		}
+	}
+
+	public static void showSpineNetworks(List<RoutesOnNetwork> allRoutes)
+	{
+		allSpineRoutes = allRoutes;
+	}
+
+	/**
+	 * Method to tell whether networks are being rewired for least-distance.
+	 * @return true if networks are being rewired for least-distance.
+	 */
+	public static boolean isShowingSpines() { return debugDialog != null && debuggingType == DebugType.SHOWSPINES; }
+
+
 	/************************************* REWIRE NETWORKS *************************************/
 
 	private static void rewireNets()
@@ -591,18 +734,49 @@ public class RoutingDebug
 		EditWindow wnd = EditWindow.getCurrent();
 		Cell cell = wnd.getCell();
 		if (cell == null) return;
+		highlighter = wnd.getRulerHighlighter();
+		new RewireNets(cell);
+	}
 
-		// get list of selected nets
-		List<ArcInst> selected = new ArrayList<ArcInst>();
-		for(Iterator<ArcInst> it = cell.getArcs(); it.hasNext(); )
+	public static class RewireNets extends Job
+	{
+		private Cell cell;
+		private EditingPreferences ep;
+
+		public RewireNets(Cell c)
 		{
-			ArcInst ai = it.next();
-			if (ai.getProto() == Generic.tech().unrouted_arc) selected.add(ai);
+			super("Rewire Networks", User.getUserTool(), Job.Type.CHANGE, null, null, Job.Priority.USER);
+			cell = c;
+			ep = UserInterfaceMain.getEditingPreferences();
+			startJob();
 		}
-		if (selected.isEmpty()) return;
 
-		debuggingType = DebugType.REWIRENETS;
-		SeaOfGatesHandlers.startInJob(cell, selected, SeaOfGatesEngineType.defaultVersion, Save.SAVE_PERIODIC);
+		public boolean doIt() throws JobException
+		{
+			// get list of selected nets
+			List<ArcInst> selected = new ArrayList<ArcInst>();
+			for(Iterator<ArcInst> it = cell.getArcs(); it.hasNext(); )
+			{
+				ArcInst ai = it.next();
+				if (ai.getProto() == Generic.tech().unrouted_arc) selected.add(ai);
+			}
+			if (selected.isEmpty()) return true;
+
+			debuggingType = DebugType.REWIRENETS;
+			endADebug = true;
+			debugDialog.router = SeaOfGatesEngineFactory.createSeaOfGatesEngine(SeaOfGatesEngineType.defaultVersion);
+			SeaOfGatesOptions prefs = new SeaOfGatesOptions();
+			prefs.getOptionsFromPreferences(false);
+			debugDialog.router.setPrefs(prefs);
+			debugDialog.svInfo = new HashMap<SearchVertex,SVState>();
+			netNames = new HashMap<Integer,String>();
+			svOrder = 1;
+
+			Job job = Job.getRunningJob();
+			debugDialog.router.routeIt(SeaOfGatesHandlers.getDefault(cell, debugDialog.router.getPrefs().resultCellName,
+				debugDialog.router.getPrefs().contactPlacementAction, job, ep), cell, false, selected);
+			return true;
+		}
 	}
 
 	/**
@@ -642,6 +816,16 @@ public class RoutingDebug
 		public RoutingDialog()
 		{
 			super(TopLevel.getCurrentJFrame());
+
+			// detect space key
+	        final String SPACE_KEY = "space-key";
+			KeyStroke space = KeyStroke.getKeyStroke(KeyEvent.VK_SPACE, 0);
+			getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(space, SPACE_KEY);
+			getRootPane().getActionMap().put(SPACE_KEY, new AbstractAction()
+			{
+				public void actionPerformed(ActionEvent event) { spacePressed(); }
+			});
+			
 			seeSV = new SearchVertex[6];
 			currentSV = null;
 			tech = Technology.getCurrent();
@@ -766,6 +950,7 @@ public class RoutingDebug
 			doGlobalRouting.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent evt) { doGlobalRouting(); } });
 			gbc = new GridBagConstraints();
 			gbc.gridx = 0;   gbc.gridy = yPos;
+			gbc.weightx = 0.3;
 			gbc.insets = new Insets(4, 4, 4, 4);
 			getContentPane().add(doGlobalRouting, gbc);
 
@@ -773,10 +958,18 @@ public class RoutingDebug
 			rewire.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent evt) { rewireNets(); } });
 			gbc = new GridBagConstraints();
 			gbc.gridx = 1;   gbc.gridy = yPos;
-			gbc.weightx = 0.5;
+			gbc.weightx = 0.3;
 			gbc.insets = new Insets(4, 4, 4, 4);
 			getContentPane().add(rewire, gbc);
 
+			JButton showSpines = new JButton("Show Spines");
+			showSpines.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent evt) { showSpines(); } });
+			gbc = new GridBagConstraints();
+			gbc.gridx = 2;   gbc.gridy = yPos;
+			gbc.weightx = 0.3;
+			gbc.insets = new Insets(4, 4, 4, 4);
+			getContentPane().add(showSpines, gbc);
+
 			yPos++;
 
 			JPanel grPanel = new JPanel();
@@ -927,7 +1120,7 @@ public class RoutingDebug
 				dirData[i].setEditable(false);
 				dirData[i].setCursor(null);
 				dirData[i].setOpaque(false);
-				dirData[i].setFocusable(false);
+				dirData[i].setFocusable(true);
 				dirData[i].setLineWrap(true);
 				dirData[i].setFont(UIManager.getFont("Label.font"));
 				gbc = new GridBagConstraints();
@@ -1013,6 +1206,7 @@ public class RoutingDebug
 				SeaOfGatesCell.showGrid(wnd, h, coords, hor, wnd.getScale(), -1);
 			}
 			h.finished();
+			wnd.repaint();
 		}
 
 		/**
@@ -1109,7 +1303,7 @@ public class RoutingDebug
 			currentSV = sv;
 			SVState svs = svInfo.get(sv);
 			svs.setBackgroundColor(Color.WHITE, h);
-			
+
 			for(int i=0; i<6; i++)
 			{
 				dirData[i].setText("");
@@ -1200,6 +1394,75 @@ public class RoutingDebug
 
 		protected void escapePressed() { endDebugging(); }
 
+		/**
+		 * Method to respond when the space key is pressed.
+		 * Finds highlighted text and shows those coordinates.
+		 */
+		protected void spacePressed()
+		{
+			Component comp = getFocusOwner();
+			if (!(comp instanceof JTextArea)) return;
+			JTextArea ta = (JTextArea)comp;
+			int start = ta.getSelectionStart();
+			int end = ta.getSelectionEnd();
+			String addr = ta.getText().substring(start, end).toLowerCase();
+
+			// parse coordinate
+			int commaPos = addr.indexOf(',');
+			if (commaPos >= 0)
+			{
+				double x = TextUtils.atofDistance(addr.substring(0, commaPos));
+				double y = TextUtils.atofDistance(addr.substring(commaPos+1));
+				Point2D p1 = new Point2D.Double(x-5, y-5);
+				Point2D p2 = new Point2D.Double(x-5, y+5);
+				Point2D p3 = new Point2D.Double(x+5, y+5);
+				Point2D p4 = new Point2D.Double(x+5, y-5);
+
+				EditWindow wnd = EditWindow.getCurrent();
+				Cell cell = wnd.getCell();
+				Highlighter h = wnd.getHighlighter();
+				h.clear();
+				h.addLine(p1, p3, cell, true, Color.RED, true);
+				h.addLine(p2, p4, cell, true, Color.RED, true);
+				h.finished();
+				wnd.repaint();
+				return;
+			}
+
+			// parse rectangle
+			int andPos = addr.indexOf(" and ");
+			int xPos = addr.indexOf("<=x<=");
+			int yPos = addr.indexOf("<=y<=");
+			if (andPos >= 0 && xPos >= 0 && yPos >= 0)
+			{
+				double lX = TextUtils.atofDistance(addr.substring(0, xPos));
+				double hX = TextUtils.atofDistance(addr.substring(xPos+5, andPos));
+				double lY = TextUtils.atofDistance(addr.substring(andPos+5, yPos));
+				double hY = TextUtils.atofDistance(addr.substring(yPos+5));
+				Point2D p1 = new Point2D.Double(lX, lY);
+				Point2D p2 = new Point2D.Double(lX, hY);
+				Point2D p3 = new Point2D.Double(hX, hY);
+				Point2D p4 = new Point2D.Double(hX, lY);
+
+				EditWindow wnd = EditWindow.getCurrent();
+				Cell cell = wnd.getCell();
+				Highlighter h = wnd.getHighlighter();
+				h.clear();
+				h.addLine(p1, p2, cell, true, Color.RED, true);
+				h.addLine(p2, p3, cell, true, Color.RED, true);
+				h.addLine(p3, p4, cell, true, Color.RED, true);
+				h.addLine(p4, p1, cell, true, Color.RED, true);
+				h.addLine(p1, p3, cell, true, Color.RED, true);
+				h.addLine(p2, p4, cell, true, Color.RED, true);
+				h.finished();
+				wnd.repaint();
+				return;
+			}
+
+			// don't know how to parse
+			Job.getUserInterface().showInformationMessage("CANNOT PARSE: "+addr, "SELECTION ERROR");
+		}
+
 		private void showGlobalRoutingGrid()
 		{
 			// show grid lines around buckets
@@ -1256,12 +1519,12 @@ public class RoutingDebug
 								if (w.getPathBucket(i-1).getBounds().getCenterX() == x)
 								{
 									if (w.getPathBucket(i-1) == n1) x = p1.getX();
-									if (w.getPathBucket(i-1) == n2) x = p2.getX();	
+									if (w.getPathBucket(i-1) == n2) x = p2.getX();
 								}
 								if (w.getPathBucket(i-1).getBounds().getCenterY() == y)
 								{
 									if (w.getPathBucket(i-1) == n1) y = p1.getY();
-									if (w.getPathBucket(i-1) == n2) y = p2.getY();	
+									if (w.getPathBucket(i-1) == n2) y = p2.getY();
 								}
 							}
 							if (i < w.getNumPathElements()-1)
@@ -1269,12 +1532,12 @@ public class RoutingDebug
 								if (w.getPathBucket(i+1).getBounds().getCenterX() == x)
 								{
 									if (w.getPathBucket(i+1) == n1) x = p1.getX();
-									if (w.getPathBucket(i+1) == n2) x = p2.getX();	
+									if (w.getPathBucket(i+1) == n2) x = p2.getX();
 								}
 								if (w.getPathBucket(i+1).getBounds().getCenterY() == y)
 								{
 									if (w.getPathBucket(i+1) == n1) y = p1.getY();
-									if (w.getPathBucket(i+1) == n2) y = p2.getY();	
+									if (w.getPathBucket(i+1) == n2) y = p2.getY();
 								}
 							}
 							for(;;)
@@ -1317,8 +1580,9 @@ public class RoutingDebug
 		 * @param cell the Cell in which routing happened.
 		 * @param h the Highlighter for showing the result.
 		 */
-		private void showPathToGoal(SearchVertex result, Cell cell, Highlighter h)
+		private void showPathToGoal(NeededRoute nr, SearchVertex result, Cell cell, Highlighter h)
 		{
+			EPoint goalCoord = null;
 			if (result == SeaOfGatesEngine.svAborted)
 			{
 				routeResult.setText("Result: Aborted by user");
@@ -1333,6 +1597,7 @@ public class RoutingDebug
 				routeResult.setText("Result: Success!");
 				SVState svs = ensureDebuggingShadow(result, false);
 				svs.changeLabel("!!GOAL!!", h);
+				goalCoord = EPoint.fromLambda(result.getX(), result.getY());
 				for(;;)
 				{
 					SearchVertex svLast = result.getLast();
@@ -1344,9 +1609,9 @@ public class RoutingDebug
 						double lowOff = lowZ * layerOffset;
 						double highOff = highZ * layerOffset;
 						h.addLine(EPoint.fromLambda(result.getX()+lowOff, result.getY()+lowOff+goalWidth),
-								EPoint.fromLambda(result.getX()+highOff-goalWidth, result.getY()+highOff), cell, true, Color.WHITE, false);
+							EPoint.fromLambda(result.getX()+highOff-goalWidth, result.getY()+highOff), cell, true, Color.WHITE, false);
 						h.addLine(EPoint.fromLambda(result.getX()+lowOff+goalWidth, result.getY()+lowOff),
-								EPoint.fromLambda(result.getX()+highOff, result.getY()+highOff-goalWidth), cell, true, Color.WHITE, false);
+							EPoint.fromLambda(result.getX()+highOff, result.getY()+highOff-goalWidth), cell, true, Color.WHITE, false);
 					} else
 					{
 						double off = result.getZ() * layerOffset;
@@ -1354,21 +1619,32 @@ public class RoutingDebug
 						{
 							// horizontal line
 							h.addLine(EPoint.fromLambda(result.getX()+off, result.getY()+off-goalWidth),
-									EPoint.fromLambda(svLast.getX()+off, svLast.getY()+off-goalWidth), cell, true, Color.WHITE, false);
+								EPoint.fromLambda(svLast.getX()+off, svLast.getY()+off-goalWidth), cell, true, Color.WHITE, false);
 							h.addLine(EPoint.fromLambda(result.getX()+off, result.getY()+off+goalWidth),
-									EPoint.fromLambda(svLast.getX()+off, svLast.getY()+off+goalWidth), cell, true, Color.WHITE, false);
+								EPoint.fromLambda(svLast.getX()+off, svLast.getY()+off+goalWidth), cell, true, Color.WHITE, false);
 						} else
 						{
 							// vertical line
 							h.addLine(EPoint.fromLambda(result.getX()+off-goalWidth, result.getY()+off),
-									EPoint.fromLambda(svLast.getX()+off-goalWidth, svLast.getY()+off), cell, true, Color.WHITE, false);
+								EPoint.fromLambda(svLast.getX()+off-goalWidth, svLast.getY()+off), cell, true, Color.WHITE, false);
 							h.addLine(EPoint.fromLambda(result.getX()+off+goalWidth, result.getY()+off),
-									EPoint.fromLambda(svLast.getX()+off+goalWidth, svLast.getY()+off), cell, true, Color.WHITE, false);
+								EPoint.fromLambda(svLast.getX()+off+goalWidth, svLast.getY()+off), cell, true, Color.WHITE, false);
 						}
 					}
 					result = svLast;
 				}
 			}
+
+			// show the possible endpoints
+			PossibleEndpoints endChoices = endADebug ? nr.getBPossibleEndpoints() :  nr.getAPossibleEndpoints();
+			for(PossibleEndpoint pe : endChoices.getEndpoints())
+			{
+				EPoint pt = pe.getCoord();
+				if (goalCoord != null && pt.getX() == goalCoord.getX() && pt.getY() == goalCoord.getY()) continue;
+				EPoint ptOff = EPoint.fromLambda(pt.getX() + possibleGoalOffset, pt.getY() + possibleGoalOffset);
+				h.addLine(EPoint.fromLambda(ptOff.getX(), ptOff.getY()), EPoint.fromLambda(pt.getX(), pt.getY()), cell, true, Color.BLACK, false);
+				h.addMessage(cell, "G", ptOff);
+			}
 		}
 
 //		private void showAandB(ArcInst ai)
@@ -1386,10 +1662,29 @@ public class RoutingDebug
 		{
 			PortInst piF = wf.getFromPortInst();
 			PortInst piT = wf.getToPortInst();
+			NeededRoute nr = wf.getNeededRoute();
+			boolean aToB = wf.isAtoB();
+			double fromTaperWidth, toTaperWidth, fromTaperLength, toTaperLength;
+			if (aToB)
+			{
+				fromTaperWidth = nr.getATaperWidth();
+				toTaperWidth = nr.getBTaperWidth();
+				fromTaperLength = nr.getATaperLength();
+				toTaperLength = nr.getBTaperLength();
+			} else
+			{
+				fromTaperWidth = nr.getBTaperWidth();
+				toTaperWidth = nr.getATaperWidth();
+				fromTaperLength = nr.getBTaperLength();
+				toTaperLength = nr.getATaperLength();
+			}
+
 			String fromMsg = "FROM: (" + TextUtils.formatDistance(wf.getFromX()) + "," + TextUtils.formatDistance(wf.getFromY()) +
 				", " + SeaOfGatesEngine.describeMetal(wf.getFromZ(), wf.getFromMask()) + "): port " + piF.getPortProto().getName() + " of node " + piF.getNodeInst().describe(false);
-			String toMsg = "TO: (" + TextUtils.formatDistance(wf.getToX()) + "," + TextUtils.formatDistance(wf.getToY()) +
+			if (fromTaperLength > 0) fromMsg += ", Taper width is " + TextUtils.formatDistance(fromTaperWidth);
+			String toMsg = "TO: (" + TextUtils.formatDistance(wf.getTo().getCenterX()) + "," + TextUtils.formatDistance(wf.getTo().getCenterY()) +
 				", " + SeaOfGatesEngine.describeMetal(wf.getToZ(), wf.getToMask()) + "): port " + piT.getPortProto().getName() + " of node " + piT.getNodeInst().describe(false);
+			if (toTaperLength > 0) toMsg += ", Taper width is " + TextUtils.formatDistance(toTaperWidth);
 			setRouteDescription(fromMsg, toMsg);
 
 			// draw the search vertices
@@ -1418,14 +1713,14 @@ public class RoutingDebug
 							double lowOff = lowZ * layerOffset;
 							double highOff = highZ * layerOffset;
 							h.addLine(EPoint.fromLambda(sv.getX()+lowOff, sv.getY()+lowOff),
-									EPoint.fromLambda(sv.getX()+highOff, sv.getY()+highOff), cell, true, Color.WHITE, false);
+								EPoint.fromLambda(sv.getX()+highOff, sv.getY()+highOff), cell, true, Color.WHITE, false);
 						} else
 						{
 							// draw line in proper metal color showing the motion
 							double off = sv.getZ() * layerOffset;
 							Color col = router.getPrimaryMetalLayer(sv.getZ()).getGraphics().getColor();
 							h.addLine(EPoint.fromLambda(sv.getX()+off, sv.getY()+off),
-									EPoint.fromLambda(sv.getLast().getX()+off, sv.getLast().getY()+off), cell, false, col, false);
+								EPoint.fromLambda(sv.getLast().getX()+off, sv.getLast().getY()+off), cell, false, col, false);
 						}
 
 						// remember lowest Z coordinate at this place so that anchor line can be drawn if it is above Metal-1
@@ -1458,6 +1753,8 @@ public class RoutingDebug
 
 	public static SVState ensureDebuggingShadow(SearchVertex sv, boolean start)
 	{
+		if (debugDialog == null) return null;
+
 		SVState svs = debugDialog.svInfo.get(sv);
 		if (svs == null)
 		{
